#!/usr/bin/env python
r"""
Bands + DOS
===========

This example shows how to compute the DOS and plot a
band structure with DOS using two GSR files.
"""
from abipy.abilab import abiopen
import abipy.data as abidata

# Open the file with energies computed on a k-path in the BZ
# and extract the band structure object.
with abiopen(abidata.ref_file("si_nscf_GSR.nc")) as nscf_file:
    nscf_ebands = nscf_file.ebands

# Open the file with energies computed with a homogeneous sampling of the BZ
# and extract the band structure object.
with abiopen(abidata.ref_file("si_scf_GSR.nc")) as gs_file:
    gs_ebands = gs_file.ebands

# Compute the DOS with the Gaussian method (use default values for
# the broadening and the step of the linear mesh.
edos = gs_ebands.get_edos()

# Plot bands and DOS.
nscf_ebands.plot_with_edos(edos, e0=None, with_gaps=True)

print("nscf_ebands.efermi", nscf_ebands.fermie)
print("gs_ebands.efermi", gs_ebands.fermie)



""""Panels for phonon-related objects."""
import param
import panel as pn
import panel.widgets as pnw

from abipy.panels.core import AbipyParameterized


class PhononBandsPlotterPanel(AbipyParameterized):

    phbands_plotter_mode = pnw.Select(name="Plot Mode", value="gridplot",
                                      options=["gridplot", "combiplot", "boxplot", "combiboxplot"]) # "animate",
    phbands_plotter_units = pnw.Select(name="Units", value="eV",
                                       options=["eV", "meV", "Ha", "cm-1", "Thz"])
    phbands_plotter_btn = pnw.Button(name="Plot", button_type='primary')

    def __init__(self, plotter, **params):
        super().__init__(**params)
        self.plotter = plotter

    @param.depends("phbands_plotter_btn.clicks")
    def on_phbands_plot_btn(self):
        if self.phbands_plotter_btn.clicks == 0: return
        plot_mode = self.phbands_plotter_mode.value
        plotfunc = getattr(self.plotter, plot_mode, None)
        if plotfunc is None:
            raise ValueError("Don't know how to handle plot_mode: %s" % plot_mode)

        fig = plotfunc(units=self.phbands_plotter_units.value, **self.fig_kwargs)
        df = self.plotter.get_phbands_frame(with_spglib=True)
        return pn.Row(pn.Column(self._mp(fig), self._df(df)), sizing_mode='scale_width')

    def get_panel(self):
        """Return tabs with widgets to interact with the |PhononBandsPlotter|."""
        tabs = pn.Tabs()
        ws = pn.Column(self.phbands_plotter_mode, self.phbands_plotter_units, self.phbands_plotter_btn)
        tabs.append(("PhbandsPlotter", pn.Row(ws, self.on_phbands_plot_btn, sizing_mode='scale_width')))

        return tabs



from __future__ import print_function, division

import sys
import wx

import wx.grid as gridlib

__all__ = [
    "SimpleGrid",
    "SimpleGridFrame",
]

class CopyPasteGrid(gridlib.Grid):
    """ 
    A Copy&Paste enabled grid class
    taken from http://wxpython-users.1045709.n5.nabble.com/copy-and-pasting-selections-in-wx-grid-cells-td2353289.html
    """
    def __init__(self, parent, **kwargs):
        super(CopyPasteGrid, self).__init__(parent, **kwargs)
        wx.EVT_KEY_DOWN(self, self.OnKey)

    def selection(self):
        # Show cell selection
        # If selection is cell...
        if self.GetSelectedCells():
            print("Selected cells " + str(self.GetSelectedCells()))
        # If selection is block...
        if self.GetSelectionBlockTopLeft():
            print("Selection block top left " + str(self.GetSelectionBlockTopLeft()))
        if self.GetSelectionBlockBottomRight():
            print("Selection block bottom right " + str(self.GetSelectionBlockBottomRight()))
        
        # If selection is col...
        if self.GetSelectedCols():
            print("Selected cols " + str(self.GetSelectedCols()))
        
        # If selection is row...
        if self.GetSelectedRows():
            print("Selected rows " + str(self.GetSelectedRows()))
    
    def currentcell(self):
        # Show cursor position
        row = self.GetGridCursorRow()
        col = self.GetGridCursorCol()
        cell = (row, col)
        print("Current cell " + str(cell))
        
    def OnKey(self, event):
        # If Ctrl+C is pressed...
        if event.ControlDown() and event.GetKeyCode() == 67:
            print("Ctrl+C")
            self.selection()
            # Call copy method
            self.copy()
            
        # If Ctrl+V is pressed...
        if event.ControlDown() and event.GetKeyCode() == 86:
            print("Ctrl+V")
            self.currentcell()
            # Call paste method
            self.paste()
            
        # If Supr is pressed
        if event.GetKeyCode() == 127:
            print("Supr")
            # Call delete method
            self.delete()
            
        # Skip other Key events
        if event.GetKeyCode():
            event.Skip()
            return

    def copy(self):
        print("Copy method")
        # Number of rows and cols
        rows = self.GetSelectionBlockBottomRight()[0][0] - self.GetSelectionBlockTopLeft()[0][0] + 1
        cols = self.GetSelectionBlockBottomRight()[0][1] - self.GetSelectionBlockTopLeft()[0][1] + 1
        
        # data variable contain text that must be set in the clipboard
        data = ''
        
        # For each cell in selected range append the cell value in the data variable
        # Tabs '\t' for cols and '\r' for rows
        for r in range(rows):
            for c in range(cols):
                data = data + str(self.GetCellValue(self.GetSelectionBlockTopLeft()[0][0] + r, self.GetSelectionBlockTopLeft()[0][1] + c))
                if c < cols - 1:
                    data = data + '\t'
            data = data + '\n'
        # Create text data object
        clipboard = wx.TextDataObject()
        # Set data object value
        clipboard.SetText(data)
        # Put the data in the clipboard
        if wx.TheClipboard.Open():
            wx.TheClipboard.SetData(clipboard)
            wx.TheClipboard.Close()
        else:
            wx.MessageBox("Can't open the clipboard", "Error")
            
    def paste(self):
        print("Paste method")
        clipboard = wx.TextDataObject()
        if wx.TheClipboard.Open():
            wx.TheClipboard.GetData(clipboard)
            wx.TheClipboard.Close()
        else:
            wx.MessageBox("Can't open the clipboard", "Error")
        data = clipboard.GetText()
        table = []
        y = -1
        # Convert text in a array of lines
        for r in data.splitlines():
            y = y +1
            x = -1
            # Convert c in a array of text separated by tab
            for c in r.split('\t'):
                x = x +1
                self.SetCellValue(self.GetGridCursorRow() + y, self.GetGridCursorCol() + x, c)
                
    def delete(self):
        print("Delete method")
        # Number of rows and cols
        rows = self.GetSelectionBlockBottomRight()[0][0] - self.GetSelectionBlockTopLeft()[0][0] + 1
        cols = self.GetSelectionBlockBottomRight()[0][1] - self.GetSelectionBlockTopLeft()[0][1] + 1
        # Clear cells contents
        for r in range(rows):
            for c in range(cols):
                self.SetCellValue(self.GetSelectionBlockTopLeft()[0][0] + r, self.GetSelectionBlockTopLeft()[0][1] + c, '')
   


#class SimpleGrid(gridlib.Grid): 
class SimpleGrid(CopyPasteGrid):

    def __init__(self, parent, table, row_labels=None, col_labels=None, **kwargs):
        """
        Args:
            parent:
                parent window.
            table:
                List of string lists.
            row_labels:
                List of strings used to name the rows.
            col_labels:
                List of strings used to name the col.
        """
        super(SimpleGrid, self).__init__(parent, id=-1, **kwargs)
        self.log = sys.stdout

        self.moveTo = None
        self.Bind(wx.EVT_IDLE, self.OnIdle)
        
        self.nrows = nrows = len(table)
        dims = {len(row) for row in table}
        if len(dims) == 1:
            self.ncols = ncols = list(dims)[0]
        else:
            raise ValueError("Each row must have the same number of columns but dims %s" % str(dims))

        self.CreateGrid(nrows, ncols)
        
        attr = gridlib.GridCellAttr()
        attr.SetTextColour(wx.BLACK)
        attr.SetBackgroundColour(wx.WHITE)
        attr.SetFont(wx.Font(14, wx.SWISS, wx.NORMAL, wx.NORMAL))

        self.SetGridCellAttr(attr)

        if row_labels is not None:
            assert len(row_labels) == nrows
            for i, label in enumerate(row_labels):
                self.SetRowLabelValue(i, label)

        if col_labels is not None:
            assert len(col_labels) == ncols
            for i, label in enumerate(col_labels):
                self.SetColLabelValue(i, label)
            self.SetColLabelAlignment(wx.ALIGN_LEFT, wx.ALIGN_BOTTOM)

        # Cell formatting
        for r, row in enumerate(table):
            for c, col in enumerate(row):
                self.SetCellValue(r, c, table[r][c])
                self.SetReadOnly(r, c, True)

        self.AutoSize()
        self.ForceRefresh()

        # test all the events
        self.Bind(gridlib.EVT_GRID_CELL_LEFT_CLICK, self.OnCellLeftClick)
        self.Bind(gridlib.EVT_GRID_CELL_RIGHT_CLICK, self.OnCellRightClick)
        self.Bind(gridlib.EVT_GRID_CELL_LEFT_DCLICK, self.OnCellLeftDClick)
        self.Bind(gridlib.EVT_GRID_CELL_RIGHT_DCLICK, self.OnCellRightDClick)

        self.Bind(gridlib.EVT_GRID_LABEL_LEFT_CLICK, self.OnLabelLeftClick)
        self.Bind(gridlib.EVT_GRID_LABEL_RIGHT_CLICK, self.OnLabelRightClick)
        self.Bind(gridlib.EVT_GRID_LABEL_LEFT_DCLICK, self.OnLabelLeftDClick)
        self.Bind(gridlib.EVT_GRID_LABEL_RIGHT_DCLICK, self.OnLabelRightDClick)

        self.Bind(gridlib.EVT_GRID_ROW_SIZE, self.OnRowSize)
        self.Bind(gridlib.EVT_GRID_COL_SIZE, self.OnColSize)

        self.Bind(gridlib.EVT_GRID_RANGE_SELECT, self.OnRangeSelect)
        self.Bind(gridlib.EVT_GRID_CELL_CHANGE, self.OnCellChange)
        self.Bind(gridlib.EVT_GRID_SELECT_CELL, self.OnSelectCell)

        self.Bind(gridlib.EVT_GRID_EDITOR_SHOWN, self.OnEditorShown)
        self.Bind(gridlib.EVT_GRID_EDITOR_HIDDEN, self.OnEditorHidden)
        self.Bind(gridlib.EVT_GRID_EDITOR_CREATED, self.OnEditorCreated)

    def SetGridCellAttr(self, attr):
        """
        Set cell attributes for the whole row.

        Args:
            attr:
                `gridlib.GridCellAttr`
        """
        # Note that GridCellAttr objects are reference counted, so attr.IncRef 
        # should be called every time Grid.Set*Attr(attr) is called. This is 
        # required to keep the Grid.Delete* methods from unexpectedly deleting the 
        # GridCellAttr object. 
        for row in range(self.nrows):
            attr.IncRef()
            self.SetRowAttr(row, attr)
        self.AutoSize()
        self.ForceRefresh()

    def OnCellLeftClick(self, evt):
        self.log.write("OnCellLeftClick: (%d,%d) %s\n" % (evt.GetRow(), evt.GetCol(), evt.GetPosition()))
        evt.Skip()

    def OnCellRightClick(self, evt):
        self.log.write("OnCellRightClick: (%d,%d) %s\n" % (evt.GetRow(), evt.GetCol(), evt.GetPosition()))
        evt.Skip()

    def OnCellLeftDClick(self, evt):
        self.log.write("OnCellLeftDClick: (%d,%d) %s\n" % (evt.GetRow(), evt.GetCol(), evt.GetPosition()))
        evt.Skip()

    def OnCellRightDClick(self, evt):
        self.log.write("OnCellRightDClick: (%d,%d) %s\n" % (evt.GetRow(), evt.GetCol(), evt.GetPosition()))
        evt.Skip()

    def OnLabelLeftClick(self, evt):
        self.log.write("OnLabelLeftClick: (%d,%d) %s\n" % (evt.GetRow(), evt.GetCol(), evt.GetPosition()))
        evt.Skip()

    def OnLabelRightClick(self, evt):
        self.log.write("OnLabelRightClick: (%d,%d) %s\n" % (evt.GetRow(), evt.GetCol(), evt.GetPosition()))
        evt.Skip()

    def OnLabelLeftDClick(self, evt):
        self.log.write("OnLabelLeftDClick: (%d,%d) %s\n" % (evt.GetRow(), evt.GetCol(), evt.GetPosition()))
        evt.Skip()

    def OnLabelRightDClick(self, evt):
        self.log.write("OnLabelRightDClick: (%d,%d) %s\n" % (evt.GetRow(), evt.GetCol(), evt.GetPosition()))
        evt.Skip()

    def OnRowSize(self, evt):
        self.log.write("OnRowSize: row %d, %s\n" % (evt.GetRowOrCol(), evt.GetPosition()))
        evt.Skip()

    def OnColSize(self, evt):
        self.log.write("OnColSize: col %d, %s\n" % (evt.GetRowOrCol(), evt.GetPosition()))
        evt.Skip()

    def OnRangeSelect(self, evt):
        if evt.Selecting():
            msg = 'Selected'
        else:
            msg = 'Deselected'
        self.log.write("OnRangeSelect: %s  top-left %s, bottom-right %s\n" %
                           (msg, evt.GetTopLeftCoords(), evt.GetBottomRightCoords()))
        evt.Skip()


    def OnCellChange(self, evt):
        self.log.write("OnCellChange: (%d,%d) %s\n" % (evt.GetRow(), evt.GetCol(), evt.GetPosition()))

        # Show how to stay in a cell that has bad data.  We can't just
        # call SetGridCursor here since we are nested inside one so it
        # won't have any effect.  Instead, set coordinates to move to in
        # idle time.
        value = self.GetCellValue(evt.GetRow(), evt.GetCol())

        if value == 'no good':
            self.moveTo = evt.GetRow(), evt.GetCol()

    def OnIdle(self, evt):
        if self.moveTo is not None:
            self.SetGridCursor(self.moveTo[0], self.moveTo[1])
            self.moveTo = None

        evt.Skip()

    def OnSelectCell(self, evt):
        if evt.Selecting():
            msg = 'Selected'
        else:
            msg = 'Deselected'
        self.log.write("OnSelectCell: %s (%d,%d) %s\n" % (msg, evt.GetRow(), evt.GetCol(), evt.GetPosition()))

        # Another way to stay in a cell that has a bad value...
        row = self.GetGridCursorRow()
        col = self.GetGridCursorCol()

        if self.IsCellEditControlEnabled():
            self.HideCellEditControl()
            self.DisableCellEditControl()

        value = self.GetCellValue(row, col)

        if value == 'no good 2':
            return  # cancels the cell selection

        evt.Skip()

    def OnEditorShown(self, evt):
        if evt.GetRow() == 6 and evt.GetCol() == 3 and \
           wx.MessageBox("Are you sure you wish to edit this cell?",
                        "Checking", wx.YES_NO) == wx.NO:
            evt.Veto()
            return

        self.log.write("OnEditorShown: (%d,%d) %s\n" % (evt.GetRow(), evt.GetCol(), evt.GetPosition()))
        evt.Skip()

    def OnEditorHidden(self, evt):
        if evt.GetRow() == 6 and evt.GetCol() == 3 and \
           wx.MessageBox("Are you sure you wish to  finish editing this cell?",
                        "Checking", wx.YES_NO) == wx.NO:
            evt.Veto()
            return

        self.log.write("OnEditorHidden: (%d,%d) %s\n" % (evt.GetRow(), evt.GetCol(), evt.GetPosition()))
        evt.Skip()

    def OnEditorCreated(self, evt):
        self.log.write("OnEditorCreated: (%d, %d) %s\n" % (evt.GetRow(), evt.GetCol(), evt.GetControl()))

    #def InstallGridHint(grid, rowcolhintcallback):
    #    prev_rowcol = [None,None]
    #    def OnMouseMotion(evt):
    #        # evt.GetRow() and evt.GetCol() would be nice to have here,
    #        # but as this is a mouse event, not a grid event, they are not
    #        # available and we need to compute them by hand.
    #        x, y = grid.CalcUnscrolledPosition(evt.GetPosition())
    #        row = grid.YToRow(y)
    #        col = grid.XToCol(x)
    #        if (row,col) != prev_rowcol and row >= 0 and col >= 0:
    #            prev_rowcol[:] = [row,col]
    #            hinttext = rowcolhintcallback(row, col)
    #            if hinttext is None:
    #                hinttext = ''
    #            grid.GetGridWindow().SetToolTipString(hinttext)
    #        evt.Skip()
    #    wx.EVT_MOTION(grid.GetGridWindow(), OnMouseMotion)


class SimpleGridFrame(wx.Frame):
    def __init__(self, parent, table, row_labels=None, col_labels=None, labels_from_table=False, **kwargs):
        """
        Args:
            labels_from_table:
                If True row_labes and col_labels are taken from the table. Not compatible with `row_labels` and `col_labels`.
        """
        super(SimpleGridFrame, self).__init__(parent, -1, **kwargs)

        if labels_from_table:
            # Extract labes from table and extract the subtable.
            assert not row_labels and not col_labels
            col_labels = table[0][1:]
            row_labels, new_table = [], []
            for row in table[1:]:
                row_labels.append(row[0])
                new_table.append(row[1:])
            table = new_table

        self.makeToolBar()

        self.panel = panel = wx.Panel(self, -1)
        self.grid = SimpleGrid(panel, table, row_labels=row_labels, col_labels=col_labels)

        self.main_sizer = main_sizer = wx.BoxSizer(wx.VERTICAL)
        main_sizer.Add(self.grid, 1, wx.EXPAND | wx.ALL, 5)

        panel.SetSizerAndFit(main_sizer)

    def makeToolBar(self):
        """Creates the tool bar."""
        self.toolbar = toolbar = self.CreateToolBar()

        self.font_picker = wx.FontPickerCtrl(toolbar, -1)
        toolbar.AddControl(control=self.font_picker) 
        toolbar.Realize()

        #menu_bar = wx.MenuBar()
        # Associate menu/toolbar items with their handlers.
        #self.ID_ = wx.NewId()
        #menu_handlers = [
        #    (self.ID_FONT_CHANGED, self.onFontChanged),
        #]
        #for combo in menu_handlers:
        #    mid, handler = combo[:2]
        #    self.Bind(wx.EVT_MENU, handler, id=mid)

        self.Bind(wx.EVT_FONTPICKER_CHANGED, self.OnFontPickerChanged, self.font_picker)

    def OnFontPickerChanged(self, event):
        """Change the Font."""
        font = self.font_picker.GetSelectedFont()
        attr = gridlib.GridCellAttr()
        attr.SetFont(font)
        self.grid.SetGridCellAttr(attr)
        self.main_sizer.Fit(self.panel)


#class DemoSimpleGrid(wx.Frame):
#    def __init__(self, parent):
#        wx.Frame.__init__(self, parent, -1, "Simple Grid Demo")
#
#        row_labels = ["row1", "row2"]
#        col_labels = ["col1", "col2", "col3"]
#
#        table = [ 
#            ["1", "2", "3"],
#            ["4", "5", "6"]
#        ]
#        self.grid = SimpleGrid(self, table, row_labels, col_labels)
#
#
#if __name__ == '__main__':
#    app = wx.App()
#    frame = DemoSimpleGrid(None)
#    frame.Show(True)
#    app.MainLoop()



"""Frames for text visualization."""

import os
import wx
import tempfile
import wx.lib.agw.flatnotebook as fnb
import abipy.gui.awx as awx

from monty.fnmatch import WildCard
from wx.py.editor import EditorFrame, EditorNotebookFrame

__all__ = [
    "SimpleTextViewer",
    "TextNotebookFrame",
]


def add_size(kwargs, size=(800, 600)):
    """Add size to kwargs if not present."""
    if "size" not in kwargs:
        kwargs["size"] = size

    return kwargs


class SimpleTextViewer(awx.Frame):
    """Very simple frame that displays text (string ) in read-only mode."""
    def __init__(self, parent, text, **kwargs):
        super(SimpleTextViewer, self).__init__(parent, **add_size(kwargs))
        wx.TextCtrl(self, -1, text, style=wx.TE_MULTILINE | wx.TE_LEFT | wx.TE_READONLY)


class MyEditorFrame(EditorFrame):
    def __init__(self, parent, filename, **kwargs):
        super(MyEditorFrame, self).__init__(parent, filename=filename, **add_size(kwargs))

    @classmethod
    def from_text(cls, parent, text, **kwargs):
        """Hack so that we can open a string in the Editor."""
        fd, filename = tempfile.mkstemp(text=True)

        with open(filename, "w") as fh:
            fh.write(text)

        return cls(parent, filename, **kwargs)


class TextNotebookFrame(awx.Frame):
    """
    This frame receives a list of strings and displays them in a notebook (read-only mode)
    """
    def __init__(self, parent, text_list, page_names, num_dirs=2, **kwargs):
        """
        Args:
            parent:
                Parent Widget.
            text_list:
                List of strings. Each string is displayed in its own page in the notebook.
            page_names:
                List of strings giving the name of the tab for each string in text_list.
            num_dirs:
                Maximum number of directories that will be shown in the tab.
        """
        super(TextNotebookFrame, self).__init__(parent, **add_size(kwargs))

        # Add the pages to the notebook with the name to show on the tab
        if not isinstance(text_list, (list, tuple)):
            text_list = [text_list]

        if not isinstance(page_names, (list, tuple)):
            page_names = [page_names]

        assert len(page_names) == len(text_list)

        # Here we create a panel and a notebook on the panel
        nb_panel = awx.Panel(self)

        try:
            style = fnb.FNB_X_ON_TAB | fnb.FNB_NAV_BUTTONS_WHEN_NEEDED
        except AttributeError:
            style = fnb.FNB_X_ON_TAB

        nb = fnb.FlatNotebook(nb_panel, style=style)

        for page_name, text in zip(page_names, text_list):
            page = wx.TextCtrl(nb, -1, text, style=wx.TE_MULTILINE|wx.TE_LEFT|wx.TE_READONLY)

            if num_dirs > 0:
                tokens = page_name.split(os.path.sep)
                page_name = os.path.join(*tokens[-num_dirs:])

            nb.AddPage(page, text=page_name)

        # Finally, put the notebook in a sizer for the nb_panel to manage the layout
        sizer = wx.BoxSizer()
        sizer.Add(nb, 1, wx.EXPAND)

        nb_panel.SetSizerAndFit(sizer)

    @classmethod
    def from_files_and_dir(cls, parent, filenames=None, dirpath=None, walk=True, wildcard=""):
        """
        Static constructure that reads the content of the files/directory specified in input.

        Args:
            filenames:
                List of files to show in the botebook. Defaults to an empty list.
            dirpath:
                Directory to scan for additional files.
            walk:
                Used only if dirpath is not None.
                If True, we scan all the files contained within dirpath and
                we add them to the list if their name match the regular expression
                given in wildcard.
            wildcard:
                String with regular expressions separated by `|`.
                Only the files matching one of the regular expressions will be showed.
                example: wildcard='*.nc|*.txt' shows only the files whose extension is in ['nc', 'txt'].
        """
        wildcard = WildCard(wildcard)

        if filenames is None:
            filenames = []

        filenames = wildcard.filter(filenames)

        if dirpath is not None:
            if not walk:
                filenames += wildcard.filter(os.listdir(dirpath))

            else:
                for root, dirnames, fnames in os.walk(dirpath):
                    for fname in fnames:
                        if wildcard.match(fname):
                            filenames.append(os.path.join(root, fname))

        #frame = EditorNotebookFrame(parent)
        #frame.notebook.DeletePage(0)
        #for fname in filenames:
        #    frame.bufferCreate(filename=fname)
        #return frame

        # Open the files and read the content in a string
        text_list = []
        for fname in filenames:
            with open(fname, "r") as fh:
                # Sanitize strings: use "ignore" to skip invalid characters in .encode/.decode like
                s = fh.read().decode("utf-8", "ignore")
                text_list.append(s)

        return cls(parent, text_list, page_names=filenames)


def wxapp_showfiles(filenames=None, dirpath=None, walk=True, wildcard=None):
    """
    Standalone applications that reads the content of the files specified
    in input and show them in a notebook.

    Args:
        filenames:
            List of files to show in the notebook. Defaults to an empty list.
        dirpath:
            Directory to scan for additional files.
        walk:
            Used only if dirpath is not None.
            If True, we scan all the files contained within dirpath and
            we add them to the list if their name match the regular expression
            given in wildcard.
        wildcard:
            String with regular expressions separated by |.
            Only the files matching one of the regular expressions will be showed.
            example: wildcard="*.nc|*.txt" shows only the files whose extension is in ["nc", "txt"].

    Returns:
        `wxpython` application.
    """
    app = awx.App()
    frame = TextNotebookFrame.from_files_and_dir(None, filenames=filenames, dirpath=dirpath, walk=walk, wildcard=wildcard)
    frame.Show()
    return app


if __name__ == "__main__":
    app = awx.App()
    frame = EditorNotebookFrame()
    #frame = wx.Frame(None, -1)
    #notebook = EditorNotebook(frame)
    for filename in ["editor.py", "__init__.py"]:
        frame.bufferCreate(filename=filename)

    #Editor(frame)
    frame.Show()
    #frame = EditorFrame()
    #frame.bufferCreate(file.path)
    #frame = EditorNotebookFrame(filename=file.path)
    app.MainLoop()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# elements_gui.py

# Copyright (c) 2005-2014, Christoph Gohlke
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the copyright holders nor the names of any
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

"""Periodic Table of Elements - A user interface for elements.py.

:Author: `Christoph Gohlke <http://www.lfd.uci.edu/~gohlke/>`_

:Version: 2013.03.18

Requirements
------------

* `CPython 2.7 <http://www.python.org>`_
* `wxPython 2.8 <http://www.wxpython.org>`_
* `Elements.py 2013.03.18 <http://www.lfd.uci.edu/~gohlke/>`_

"""
from __future__ import division, print_function

import sys
import math
import io
import webbrowser

import wxversion
#wxversion.ensureMinimal('2.8')
import wx
from wx.lib import fancytext, buttons, rcsizer

from elements import ELEMENTS, SERIES

from abipy.gui.awx.panels import ListCtrlFromTable
try:
    from pseudo_dojo.refdata.nist import database as nist
except ImportError:
    pass


class MainApp(wx.App):
    """Main application."""

    name = "Periodic Table of Elements"
    version = "2012.04.05"
    website = "http://www.lfd.uci.edu/~gohlke/"
    copyright = ("Christoph Gohlke\n"
                 "Laboratory for Fluorescence Dynamics\n"
                 "University of California, Irvine")
    icon = "elements"

    def OnInit(self):
        _ = wx.LogNull()
        wx.InitAllImageHandlers()
        mainframe = WxPeriodicTable(None, -1)
        self.SetTopWindow(mainframe)
        if wx.Platform != "__WXMAC__":
            mainframe.Centre()
        mainframe.ApplyLayout(False)
        mainframe.Show()
        return 1


class ElementButton(buttons.GenToggleButton):
    """Button representing chemical element."""

    def __init__(self, *args, **kwds):
        buttons.GenToggleButton.__init__(self, *args, **kwds)
        self.color = wx.Colour(255, 255, 255)
        self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)
        self.Bind(wx.EVT_RIGHT_DOWN, self.onRightDown)

    @property
    def Z(self):
        """Atomic number corresponding to this button."""
        return self.GetId() - 100

    def OnEraseBackground(self, event):
        pass

    def SetButtonColour(self, color):
        self.color = color

    def OnPaint(self, event):
        width, height = self.GetClientSizeTuple()
        dc = wx.BufferedPaintDC(self)
        brush = wx.Brush(self.GetBackgroundColour(), wx.SOLID)
        if wx.Platform == "__WXMAC__":
            brush.MacSetTheme(1)  # kThemeBrushDialogBackgroundActive
        dc.SetBackground(brush)
        dc.SetPen(wx.TRANSPARENT_PEN)
        dc.SetBrush(wx.Brush(self.color if self.up else 'WHITE', wx.SOLID))
        dc.Clear()
        dc.DrawRectanglePointSize((1, 1), (width - 2, height - 2))
        if self.up:
            self.DrawBezel(dc, 0, 0, width - 1, height - 1)
        self.DrawLabel(dc, width, height)
        if (self.hasFocus and self.useFocusInd):
            self.DrawFocusIndicator(dc, width, height)

    def DrawLabel(self, dc, width, height):
        font = self.GetFont()
        font.SetWeight(wx.FONTWEIGHT_BOLD)
        dc.SetFont(font)

        if self.IsEnabled():
            dc.SetTextForeground(self.GetForegroundColour())
        else:
            dc.SetTextForeground(wx.SystemSettings.GetColour(
                wx.SYS_COLOUR_GRAYTEXT))

        label = self.GetLabel()
        txtwidth, txtheight = dc.GetTextExtent(label)
        xpos = (width - txtwidth) // 2
        ypos = (height*0.75 - txtheight) // 2 - 1
        dc.DrawText(label, xpos, ypos)

        font.SetWeight(wx.FONTWEIGHT_LIGHT)
        font.SetPointSize((font.GetPointSize()*6) // 8)
        dc.SetFont(font)
        label = "%i" % (self.GetId() - 100)
        txtwidth, txtheight = dc.GetTextExtent(label)
        dc.DrawText(label, (width-txtwidth)//2, 4+ypos+(height-txtheight)//2)

    def makePopupMenu(self):
        """Build and return the popup menu."""
        menu = wx.Menu()

        self.ID_POPUP_NIST_LDA = wx.NewId()
        menu.Append(self.ID_POPUP_NIST_LDA, "NIST SCF data (LDA)")

        # Associate menu/toolbar items with their handlers.
        menu_handlers = [
            (self.ID_POPUP_NIST_LDA, self.onNistLda),
        ]
                                                            
        for combo in menu_handlers:
            mid, handler = combo[:2]
            self.Bind(wx.EVT_MENU, handler, id=mid)
                                                     
        return menu

    def onRightDown(self, event):
        """Called when right button is pressed."""
        popup_menu = self.makePopupMenu()
        self.PopupMenu(popup_menu, event.GetPosition())

    def onNistLda(self, event):
        """
        Show the LDA levels of the neutral atom.
        (useful to decide if semicore states should be included in the valence).
        """
        try:
            entry = nist.get_neutral_entry(self.Z)
        except KeyError:
            return

        table = entry.to_table()
        frame = wx.Frame(self, title="LDA levels for atom %s (NIST database) " % entry.symbol)
        ListCtrlFromTable(frame, table)
        frame.Show()


class PeriodicPanel(wx.Panel):
    """Periodic table of elements panel."""

    layout = """
        .  1  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  18 .
        1  H  2  .  .  .  .  .  .  .  .  .  .  13 14 15 16 17 He .
        2  Li Be .  .  .  .  .  .  .  .  .  .  B  C  N  O  F  Ne .
        3  Na Mg 3  4  5  6  7  8  9  10 11 12 Al Si P  S  Cl Ar .
        4  K  Ca Sc Ti V  Cr Mn Fe Co Ni Cu Zn Ga Ge As Se Br Kr .
        5  Rb Sr Y  Zr Nb Mo Tc Ru Rh Pd Ag Cd In Sn Sb Te I  Xe .
        6  Cs Ba *  Hf Ta W  Re Os Ir Pt Au Hg Tl Pb Bi Po At Rn .
        7  Fr Ra ** Rf Db Sg Bh Hs Mt .  .  .  .  .  .  .  .  .  .
        .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
        .  .  .  *  La Ce Pr Nd Pm Sm Eu Gd Tb Dy Ho Er Tm Yb Lu .
        .  .  .  ** Ac Th Pa U  Np Pu Am Cm Bk Cf Es Fm Md No Lr .
    """
    # Class used to instanciates the buttons in the panel.
    element_button_class = ElementButton

    def __init__(self, *args, **kwds):
        kwds["style"] = wx.TAB_TRAVERSAL
        wx.Panel.__init__(self, *args, **kwds)
        rows = len(self.layout.splitlines()) - 2
        cols = len(self.layout.splitlines()[1].split())
        self.sizer = wx.FlexGridSizer(rows, cols, 0, 0)
        self.buttons = list(range(0, len(ELEMENTS)))
        self.selected = -1

        self.info = ElementPanel(self, -1, pos=(0, 0))

        # create element buttons
        buttonsize = math.ceil(float(self.info.GetSize()[0] + 4) / 9.0)
        if buttonsize < 30:
            buttonsize = 30
        for row in self.layout.splitlines()[1:-1]:
            for col in row.split():
                if col == '.':
                    self.sizer.Add((SPACER, SPACER))
                elif col[0] in '123456789*':
                    static = wx.StaticText(self, -1, col,
                        style=wx.ALIGN_CENTER|wx.ALIGN_BOTTOM)
                    self.sizer.Add(static, 0,
                        (wx.ALL|wx.ALIGN_CENTER_HORIZONTAL|
                         wx.ALIGN_CENTER_VERTICAL|wx.FIXED_MINSIZE), SPACER//2)
                else:
                    ele = ELEMENTS[col]
                    button = self.element_button_class(self, ele.number+100, ele.symbol,
                                           size=(buttonsize, buttonsize))
                    self.buttons[ele.number - 1] = button
                    button.SetBezelWidth(1)
                    button.SetToolTipString(ele.name)
                    col = COLORS[ele.series]
                    button.SetButtonColour(wx.Colour(col[0], col[1], col[2]))
                    self.sizer.Add(button, 0, (wx.LEFT|wx.BOTTOM|
                        wx.FIXED_MINSIZE|wx.ALIGN_CENTER_HORIZONTAL|
                        wx.ALIGN_CENTER_VERTICAL), 0)
                    self.Bind(wx.EVT_BUTTON, self.OnSelect, button)

        self.SetAutoLayout(True)
        self.SetSizer(self.sizer)
        self.sizer.SetSizeHints(self)
        self.sizer.Fit(self)

        # position element info panel
        cw = self.sizer.GetColWidths()
        rh = self.sizer.GetRowHeights()
        self.info.Move((sum(cw[:3])+cw[3]//2, (rh[0]-SPACER)//2-1))

        # legend of chemical series
        self.legendpos = (sum(cw[:13]), (rh[0]-SPACER)//2-1)
        self.legendsize = (sum(cw[13:17]) + cw[17]//2, -1)
        self.legend = wx.StaticText(self, -1, " Alkaline earth metals ",
            style=wx.ALIGN_CENTER, pos=self.legendpos, size=self.legendsize)
        self.legend.SetToolTipString("Chemical series")

        # blinking element button
        self.highlight = False
        self.timer = wx.Timer(self)
        self.timer.Start(750)
        self.Bind(wx.EVT_TIMER, self.OnTimer)

    def AddCtrl(self, ctrl, pos=200):
        self.sizer.Remove(pos)
        self.sizer.Insert(pos, ctrl, 0, (wx.ALL|wx.ALIGN_CENTER_HORIZONTAL|
                                         wx.ALIGN_BOTTOM|wx.FIXED_MINSIZE), 0)
        self.Layout()

    def OnTimer(self, evt):
        button = self.buttons[self.selected]
        button.SetToggle(button.up)

    def OnSelect(self, evt):
        self.GetParent().SetSelection(evt.GetId() - 101)

    def SetSelection(self, select):
        """Set active element."""
        if self.selected == select:
            return
        # reset old selection
        self.buttons[self.selected].SetToggle(False)
        # highlight new selection
        self.selected = select
        self.buttons[select].SetToggle(True)
        ele = ELEMENTS[select + 1]
        col = COLORS[ele.series]
        self.legend.SetBackgroundColour(wx.Colour(col[0], col[1], col[2]))
        self.legend.SetLabel(SERIES[ele.series])
        self.legend.Move(self.legendpos)
        self.legend.SetSize(self.legendsize)
        self.info.SetSelection(select)


class ElementPanel(wx.Panel):
    """Element information panel."""

    def __init__(self, *args, **kwds):
        kwds["style"] = wx.NO_BORDER | wx.TAB_TRAVERSAL
        wx.Panel.__init__(self, *args, **kwds)
        self.selected = -1

        # create controls
        self.number = wx.StaticText(self, -1, "808",
            style=wx.ALIGN_RIGHT)
        self.position = wx.StaticText(self, -1, "6, 88, 9",
            style=wx.ALIGN_LEFT)
        self.symbol = wx.StaticText(self, -1, "Mm",
            style=wx.ALIGN_CENTER_HORIZONTAL)
        self.name = wx.StaticText(self, -1, "Praseodymium ",
            style=wx.ALIGN_CENTER_HORIZONTAL)
        self.mass = wx.StaticText(self, -1, "123.4567890 ",
            style=wx.ALIGN_CENTER_HORIZONTAL)
        self.massnumber = wx.StaticText(self, -1, "123 A ",
            style=wx.ALIGN_RIGHT)
        self.protons = wx.StaticText(self, -1, "123 P ",
            style=wx.ALIGN_RIGHT)
        self.neutrons = wx.StaticText(self, -1, "123 N ",
            style=wx.ALIGN_RIGHT)
        self.electrons = wx.StaticText(self, -1, "123 e ",
            style=wx.ALIGN_RIGHT)
        self.eleshell = wx.StaticText(self, -1, "2, 8, 18, 32, 32, 15, 2",
            style=wx.ALIGN_LEFT)
        self.eleconfig = StaticFancyText(self, -1,
            "[Xe] 4f<sup>14</sup> 5d<sup>10</sup>"
            " 6s<sup>2</sup> 6p<sup>6</sup> ",
            style=wx.ALIGN_LEFT)
        self.oxistates = wx.StaticText(self, -1, "1*, 2, 3, 4, 5, 6, -7 ")
        self.atmrad = wx.StaticText(self, -1, "1.234 A ",
            style=wx.ALIGN_RIGHT)
        self.ionpot = wx.StaticText(self, -1, "123.4567890 eV ")
        self.eleneg = wx.StaticText(self, -1, "123.45678 ")

        # set control properties
        font = self.GetFont()
        font.SetWeight(wx.FONTWEIGHT_BOLD)
        self.number.SetFont(font)
        self.name.SetFont(font)
        font.SetPointSize(font.GetPointSize() * 1.9)
        self.symbol.SetFont(font)

        self.number.SetToolTipString("Atomic number")
        self.position.SetToolTipString("Group, Period, Block")
        self.symbol.SetToolTipString("Symbol")
        self.name.SetToolTipString("Name")
        self.mass.SetToolTipString("Relative atomic mass")
        self.eleshell.SetToolTipString("Electrons per shell")
        self.massnumber.SetToolTipString("Mass number (most abundant isotope)")
        self.protons.SetToolTipString("Protons")
        self.neutrons.SetToolTipString("Neutrons (most abundant isotope)")
        self.electrons.SetToolTipString("Electrons")
        self.eleconfig.SetToolTipString("Electron configuration")
        self.oxistates.SetToolTipString("Oxidation states")
        self.atmrad.SetToolTipString("Atomic radius")
        self.ionpot.SetToolTipString("Ionization potentials")
        self.eleneg.SetToolTipString("Electronegativity")

        # layout
        sizer = rcsizer.RowColSizer()
        sizer.col_w = SPACER
        sizer.row_h = SPACER
        sizer.Add(self.number, row=0, col=0,
            flag=wx.ALIGN_CENTER_HORIZONTAL|wx.FIXED_MINSIZE)
        sizer.Add(self.position, row=0, col=1,
            flag=wx.ALIGN_LEFT|wx.FIXED_MINSIZE)
        sizer.Add(self.symbol, row=1, col=0, rowspan=2, colspan=2,
            flag=(wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL|
                  wx.FIXED_MINSIZE))
        sizer.Add(self.name, row=3, col=0, colspan=2,
            flag=wx.ALIGN_CENTER_HORIZONTAL|wx.FIXED_MINSIZE)
        sizer.Add(self.mass, row=4, col=0, colspan=2,
            flag=wx.ALIGN_CENTER_HORIZONTAL|wx.FIXED_MINSIZE)
        sizer.Add(self.massnumber, row=0, col=2,
            flag=wx.ALIGN_RIGHT|wx.FIXED_MINSIZE)
        sizer.Add(self.protons, row=1, col=2,
            flag=wx.ALIGN_RIGHT|wx.FIXED_MINSIZE)
        sizer.Add(self.neutrons, row=2, col=2,
            flag=wx.ALIGN_RIGHT|wx.FIXED_MINSIZE)
        sizer.Add(self.electrons, row=3, col=2,
            flag=wx.ALIGN_RIGHT|wx.FIXED_MINSIZE)
        sizer.Add(self.atmrad, row=4, col=2,
            flag=wx.ALIGN_RIGHT|wx.FIXED_MINSIZE)
        sizer.Add(self.eleconfig, row=0, col=4, flag=wx.ADJUST_MINSIZE)
        sizer.Add(self.eleshell, row=1, col=4, flag=wx.ADJUST_MINSIZE)
        sizer.Add(self.oxistates, row=2, col=4, flag=wx.ADJUST_MINSIZE)
        sizer.Add(self.ionpot, row=3, col=4, flag=wx.ADJUST_MINSIZE)
        sizer.Add(self.eleneg, row=4, col=4, flag=wx.ADJUST_MINSIZE)

        self.SetAutoLayout(True)
        self.SetSizer(sizer)
        sizer.Fit(self)
        sizer.SetSizeHints(self)
        self.Layout()

    def SetSelection(self, select):
        """Set active element."""
        if self.selected == select:
            return
        self.Freeze()
        self.selected = select
        ele = ELEMENTS[select + 1]

        self.number.SetLabel("%i" % ele.number)
        self.position.SetLabel("%i, %i, %s" % (
            ele.group, ele.period, ele.block))
        self.mass.SetLabel("%.10g" % ele.mass)
        self.eleshell.SetLabel(', '.join("%i" % i for i in ele.eleshells))
        self.massnumber.SetLabel('%i A ' % ele.nominalmass)
        self.protons.SetLabel('%i P ' % ele.protons)
        self.neutrons.SetLabel('%i N ' % ele.neutrons)
        self.electrons.SetLabel('%i e ' % ele.electrons)
        self.oxistates.SetLabel(ele.oxistates)
        self.atmrad.SetLabel(_u("%.10g \xc5 ") % ele.atmrad if ele.atmrad else
                             "")
        self.eleneg.SetLabel("%.10g" % ele.eleneg if ele.eleneg else "")
        self.ionpot.SetLabel(
            "%.10g eV" % ele.ionenergy[0] if ele.ionenergy else "")
        self.symbol.SetLabel(ele.symbol)
        self.name.SetLabel(ele.name)

        label = []
        for orb in ele.eleconfig.split():
            if not orb.startswith('[') and len(orb) > 2:
                orb = orb[:2] + '<sup>' + orb[2:] + '</sup>'
            label.append(orb)
        label.append("<sup> </sup>")  # fix ADJUST_MINSIZE
        self.eleconfig.SetLabel(' '.join(label))

        self.Thaw()
        self.Layout()


class DetailsPanel(wx.Panel):
    """Element details panel."""

    def __init__(self, *args, **kwds):
        kwds["style"] = wx.NO_BORDER|wx.TAB_TRAVERSAL
        wx.Panel.__init__(self, *args, **kwds)
        self.selected = -1

        # create controls
        cb_style = wx.CB_READONLY|wx.CB_SORT
        if wx.Platform == "__WXMAC__":
            cb_style = wx.CB_READONLY


        self.names = LabeledCtrl(self, wx.ComboBox, "Element Name",
            choices=[p.name for p in ELEMENTS],
            style=cb_style, size=(1, -1))
        self.symbols = LabeledCtrl(self, wx.ComboBox, "Symbol", '',
            choices=[p.symbol for p in ELEMENTS],
            style=cb_style, size=(1, -1))
        self.numbers = LabeledCtrl(self, wx.ComboBox, "Number",
            choices=["%s" % p.number for p in ELEMENTS],
            style=wx.CB_READONLY, size=(1, -1))
        self.mass = LabeledCtrl(self, wx.ComboBox, "Relative Atomic Mass",
            choices=["%-.10g" % p.mass for p in ELEMENTS],
            style=wx.CB_READONLY, size=(1, -1))
        self.atmrad = LabeledCtrl(self, wx.ComboBox,
                                  _u("Atomic Radius (\xc5)"),
            choices=["%-.10g" % p.atmrad for p in ELEMENTS],
            style=wx.CB_READONLY, size=(1, -1))
        self.covrad = LabeledCtrl(self, wx.ComboBox,
                                  _u("Covalent Radius (\xc5)"),
            choices=["%-.10g" % p.covrad for p in ELEMENTS],
            style=wx.CB_READONLY, size=(1, -1))
        self.vdwrad = LabeledCtrl(self, wx.ComboBox,
                                  _u("V.d.Waals Radius (\xc5)"),
            choices=["%-.10g" % p.vdwrad for p in ELEMENTS],
            style=wx.CB_READONLY, size=(1, -1))
        self.eleneg = LabeledCtrl(self, wx.ComboBox, "Electronegativity",
            choices=["%-.10g" % p.eleneg for p in ELEMENTS],
            style=wx.CB_READONLY, size=(1, -1))
        self.eleconfig = LabeledCtrl(self, wx.ComboBox, "e- Config",
            choices=[p.eleconfig for p in ELEMENTS],
            style=wx.CB_READONLY, size=(1, -1))
        self.eleshells = LabeledCtrl(self, wx.ComboBox, "Electrons per Shell",
            choices=[', '.join("%i" % i for i in p.eleshells)
                     for p in ELEMENTS],
            style=wx.CB_READONLY, size=(1, -1))
        self.oxistates = LabeledCtrl(self, wx.ComboBox, "Oxidation States",
            choices=[p.oxistates for p in ELEMENTS],
            style=wx.CB_READONLY, size=(1, -1))
        self.ionpot = LabeledCtrl(self, wx.Choice,
            "Ionization Potentials (eV)", choices=[], size=(1, -1))
        self.isotopes = LabeledCtrl(self, wx.Choice, "Isotopes",
            choices=[], size=(1, -1))

        # layout
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer_top = wx.BoxSizer(wx.HORIZONTAL)
        sizer_left = wx.BoxSizer(wx.VERTICAL)
        sizer_right = wx.BoxSizer(wx.VERTICAL)
        sizer_num = wx.BoxSizer(wx.HORIZONTAL)
        style = wx.RIGHT|wx.BOTTOM|wx.EXPAND|wx.ADJUST_MINSIZE
        sizer_left.Add(self.names, 0, style, SPACER)
        sizer_left.Add(self.mass, 0, style, SPACER)
        sizer_left.Add(self.atmrad, 0, style, SPACER)
        sizer_left.Add(self.covrad, 0, style, SPACER)
        sizer_left.Add(self.vdwrad, 0, style, SPACER)
        sizer_left.Add(self.eleneg, 0, style, SPACER)
        sizer_top.Add(sizer_left, 1, wx.LEFT|wx.RIGHT, 0)
        style = wx.BOTTOM|wx.EXPAND|wx.ADJUST_MINSIZE
        sizer_num.Add(self.symbols, 1, style, 0)
        sizer_num.Add((SPACER, 5), 0, 0, 0)
        sizer_num.Add(self.numbers, 1, style, 0)
        sizer_right.Add(sizer_num, 0, style, SPACER)
        sizer_right.Add(self.eleconfig, 0, style, SPACER)
        sizer_right.Add(self.eleshells, 0, style, SPACER)
        sizer_right.Add(self.oxistates, 0, style, SPACER)
        sizer_right.Add(self.ionpot, 0, style, SPACER)
        sizer_right.Add(self.isotopes, 0, style, SPACER)
        sizer_top.Add(sizer_right, 1, wx.TOP|wx.RIGHT, 0)
        sizer.Add(sizer_top, 1,
            wx.LEFT|wx.RIGHT|wx.TOP|wx.EXPAND|wx.ADJUST_MINSIZE, SPACER)
        self.SetAutoLayout(True)
        self.SetSizerAndFit(sizer, True)
        sizer.SetSizeHints(self)
        self.Layout()

        # bind events
        self.Bind(wx.EVT_COMBOBOX, self.OnSelectName, self.names.ctrl)
        self.Bind(wx.EVT_COMBOBOX, self.OnSelectSymbol, self.symbols.ctrl)
        self.Bind(wx.EVT_COMBOBOX, self.OnSelect, self.numbers.ctrl)
        self.Bind(wx.EVT_COMBOBOX, self.OnSelect, self.mass.ctrl)
        self.Bind(wx.EVT_COMBOBOX, self.OnSelect, self.atmrad.ctrl)
        self.Bind(wx.EVT_COMBOBOX, self.OnSelect, self.covrad.ctrl)
        self.Bind(wx.EVT_COMBOBOX, self.OnSelect, self.vdwrad.ctrl)
        self.Bind(wx.EVT_COMBOBOX, self.OnSelect, self.eleshells.ctrl)
        self.Bind(wx.EVT_COMBOBOX, self.OnSelect, self.eleneg.ctrl)
        self.Bind(wx.EVT_COMBOBOX, self.OnSelect, self.eleconfig.ctrl)
        self.Bind(wx.EVT_COMBOBOX, self.OnSelect, self.oxistates.ctrl)

    def SetSelection(self, select):
        """Set active element."""
        if self.selected == select:
            return
        self.selected = select
        ele = ELEMENTS[select+1]

        self.names.ctrl.SetStringSelection(ele.name)
        self.symbols.ctrl.SetStringSelection(ele.symbol)
        self.numbers.ctrl.SetSelection(select)
        self.mass.ctrl.SetSelection(select)
        self.eleconfig.ctrl.SetSelection(select)
        self.atmrad.ctrl.SetSelection(select)
        self.covrad.ctrl.SetSelection(select)
        self.vdwrad.ctrl.SetSelection(select)
        self.eleneg.ctrl.SetSelection(select)
        self.eleshells.ctrl.SetSelection(select)
        self.oxistates.ctrl.SetSelection(select)

        self.isotopes.ctrl.Clear()
        for index, massnum in enumerate(sorted(ele.isotopes)):
            iso = ele.isotopes[massnum]
            self.isotopes.ctrl.Append("%3i:  %8.4f , %8.4f%%" % (
                massnum, iso.mass, iso.abundance*100.0))
            if massnum == ele.nominalmass:
                self.isotopes.ctrl.SetSelection(index)

        self.ionpot.ctrl.Clear()
        for ion in ele.ionenergy:
            self.ionpot.ctrl.Append("%8.4f" % ion)
        self.ionpot.ctrl.SetSelection(0)

    def OnSelect(self, evt):
        self.SetSelection(evt.GetSelection())
        event = SelectionEvent(pteEVT_ELE_CHANGED, self.GetId(), self.selected)
        self.GetEventHandler().ProcessEvent(event)
        evt.Skip()

    def OnSelectName(self, evt):
        name = self.names.ctrl.GetValue()
        self.SetSelection(ELEMENTS[name].number - 1)
        event = SelectionEvent(pteEVT_ELE_CHANGED, self.GetId(), self.selected)
        self.GetEventHandler().ProcessEvent(event)
        evt.Skip()

    def OnSelectSymbol(self, evt):
        name = self.symbols.ctrl.GetValue()
        self.SetSelection(ELEMENTS[name].number - 1)
        event = SelectionEvent(pteEVT_ELE_CHANGED, self.GetId(), self.selected)
        self.GetEventHandler().ProcessEvent(event)
        evt.Skip()


class DecriptionPanel(wx.Panel):
    """Element description panel."""

    def __init__(self, *args, **kwds):
        kwds["style"] = wx.NO_BORDER|wx.TAB_TRAVERSAL
        wx.Panel.__init__(self, *args, **kwds)
        self.selected = -1

        self.description = wx.TextCtrl(self, -1, " \n \n",
            style=wx.TE_MULTILINE|wx.TE_READONLY)
        font = self.description.GetFont()
        font.SetPointSize((font.GetPointSize() + 1))
        self.description.SetFont(font)
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.description, 1,
            wx.TOP|wx.LEFT|wx.RIGHT|wx.BOTTOM|wx.EXPAND|wx.FIXED_MINSIZE,
            SPACER)

        self.SetAutoLayout(True)
        self.SetSizerAndFit(sizer, True)
        sizer.SetSizeHints(self)
        self.Layout()

    def SetSelection(self, select):
        """Set active element."""
        if self.selected == select:
            return
        self.selected = select

        ele = ELEMENTS[select + 1]
        self.description.SetValue(ele.description)


class LabeledCtrl(wx.BoxSizer):
    """BoxSizer containing label, control, and unit."""

    def __init__(self, parent, control, label, unit=None, space="  ",
                 *args, **kwds):
        wx.BoxSizer.__init__(self, wx.HORIZONTAL)
        self.label = wx.StaticText(parent, -1, label + space)
        self.ctrl = control(parent, -1, *args, **kwds)
        self.Add(self.label, 0, wx.ALIGN_CENTER_VERTICAL|wx.FIXED_MINSIZE)
        self.Add(self.ctrl, 1, (wx.LEFT|wx.EXPAND|wx.ALIGN_CENTER_VERTICAL|
                                wx.ALIGN_RIGHT|wx.ADJUST_MINSIZE), 0)
        if unit:
            self.unit = wx.StaticText(parent, -1, unit)
            self.Add(self.unit, 0,
                wx.RIGHT|wx.ALIGN_CENTER_VERTICAL|wx.FIXED_MINSIZE, 0)
        else:
            self.unit = None


class StaticFancyText(fancytext.StaticFancyText):
    """StaticFancyText with SetLabel function."""

    def SetLabel(self, label):
        bmp = fancytext.RenderToBitmap(
            label, wx.Brush(self.GetBackgroundColour(), wx.SOLID))
        self.SetBitmap(bmp)


class DisclosureCtrl(buttons.GenBitmapTextToggleButton):
    """Disclosure triangle button."""

    bmp0 = (b'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00\r\x00\x00'
            b'\x00\r\x08\x06\x00\x00\x00r\xeb\xe4|\x00\x00\x00\x04sBIT'
            b'\x08\x08\x08\x08|\x08d\x88\x00\x00\x00\xd5IDAT(\x91\x9d'
            b'\x921n\x83@\x10E\xdf.!\x96\x9c\x90P"\xd9\x86\x92\x82\x03p'
            b'\x02h\xa9i9A\xe2\xb38)9\x02w\xc0\t\x94I\xa0H\xd2\xc7'
            b'\x96(\x918\xc0\xa6Z\x17\x91\x05&_\x1a\xe9k4O\xfa3\x1a!'
            b'\xa4\xc1\\Im>\xde\xdf\xd4l\xa8,K\x9e\x9fv\xeax\xf8\x99\x84'
            b'\x85\x8e\xb7}|P\x00\xb6m\x13\x04\x01q\x1cc\xdd\xdd\x8bs\xd0'
            b'\xd5\xdfF\xdf\xf7TUE\xdb\xb6\xbc\xbe\xecU\x18\x86\x98\xd7'
            b'\x0b1\ni\r\xc3@Q\x14\xd4u\xcd\xf7\xd7\xa7r]\x97\xe5\xcd'
            b'\xad\x18\x85\xb4\xba\xae#\xcfs|\xdf?\xf5\xe4\xc8<\x00\x8e'
            b'\xe3\x90e\x19i\x9aN\xc7\xb3,\x8b(\x8a\xb8h\xa7Y\xd7'
            b'\xf3<\x0f\xd34I\x92\x84\xd5zsv\xf8$!\r\x844h\x9aFi?Y\xff'
            b'\xf9\xbd_\xd7\x8c7Z\xc0k\x8d8\x00\x00\x00\x00IEND\xaeB`\x82')

    bmp1 = (b'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00\r\x00\x00'
            b'\x00\r\x08\x06\x00\x00\x00r\xeb\xe4|\x00\x00\x00\x04sBIT'
            b'\x08\x08\x08\x08|\x08d\x88\x00\x00\x00\xc1IDAT(\x91\x9d\xd2;'
            b'\x8e\xc20\x10\x80\xe1\x7fB\x0c\x95\xc1\xa5%\xc0)S\xb8I\xd2'
            b'\xb8Le\x97\xf6\x99s\x81\xdd\xbdBN\xb2\xdb!Y\t\xac`\xbay|'
            b'\xa3)F\xa49\xf0n4o\x8bOQ\x0b\xf0\xf3\xfd\xf5\xbb,\x0b\xeb'
            b'\xba>\x1d\xec\xba\x8ey\x9e\x19\xc6I\x1a\x80a\x9cD)\xf5r'
            b'\xbbR\x8aa\x9c\xa4:\xaf\x94\x821f\x17\x18c(\xa5<\xf2\x07'
            b'\xba\xde\xee\xe2\xbd\xdfE\xde{\xae\xb7\xbbl\x10@J\t\xadu\x05'
            b'\xb4\xd6\xa4\x94\xaaZ\x85\xf4\xf9"1\xc6j \xc6\x88>_\xe4)\x02'
            b'\x08!`\xad\x05\xc0ZK\x08as\xee\x06\xa9\xe3Ir\xce\xb4mK\xce'
            b'\x19u<\xc9\xbf\x08\xc09G\xdf\xf78\xe7\xf6\xda\xc8\'\xbf\xf7'
            b'\x07\x13\x12\x18B\x17\x9fx\xa0\x00\x00\x00\x00IEND\xaeB`\x82')

    def __init__(self, parent, winid, label, *args, **kwds):
        kwds["style"] = wx.BORDER_NONE|wx.BU_EXACTFIT
        buttons.GenBitmapTextToggleButton.__init__(self, parent, winid, None,
                                                   label, *args, **kwds)
        if isinstance(self.bmp0, type(b'')):
            self.__class__.bmp0 = wx.BitmapFromImage(wx.ImageFromStream(
                io.BytesIO(self.bmp0)))
            self.__class__.bmp1 = wx.BitmapFromImage(wx.ImageFromStream(
                io.BytesIO(self.bmp1)))

        self.SetBitmapLabel(self.bmp0)
        self.SetBitmapSelected(self.bmp1)
        if not label:
            self.SetSize(self.bmp0.GetSize())
        else:
            self.SetBestSize()
        self.labelDelta = 0
        self.useFocusInd = False
        self.SetToolTipString('Show')
        self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)

    def OnEraseBackground(self, event):
        pass

    def Notify(self):
        wx.lib.buttons.GenBitmapTextToggleButton.Notify(self)
        self.SetToolTipString("%s" % ('Show' if self.up else 'Hide'))

    def DoGetBestSize(self):
        width, height, usemin = self._GetLabelSize()
        return width + 5, height + 4

    def OnPaint(self, event):
        width, height = self.GetClientSizeTuple()
        dc = wx.BufferedPaintDC(self)
        brush = None
        bgcol = self.GetBackgroundColour()
        brush = wx.Brush(bgcol, wx.SOLID)
        defattr = self.GetDefaultAttributes()
        if self.style & wx.BORDER_NONE and bgcol == defattr.colBg:
            defattr = self.GetParent().GetDefaultAttributes()
            if self.GetParent().GetBackgroundColour() == defattr.colBg:
                if wx.Platform == "__WXMSW__":
                    if self.DoEraseBackground(dc):
                        brush = None
                elif wx.Platform == "__WXMAC__":
                    brush.MacSetTheme(1)
            else:
                bgcol = self.GetParent().GetBackgroundColour()
                brush = wx.Brush(bgcol, wx.SOLID)
        if brush is not None:
            dc.SetBackground(brush)
            dc.Clear()
        self.DrawLabel(dc, width, height)

    def DrawLabel(self, dc, width, height, center=False):
        bmp = self.bmpLabel
        if bmp is not None:
            if self.bmpDisabled and not self.IsEnabled():
                bmp = self.bmpDisabled
            if self.bmpFocus and self.hasFocus:
                bmp = self.bmpFocus
            if self.bmpSelected and not self.up:
                bmp = self.bmpSelected
            bmpwidth, bmpheight = bmp.GetWidth(), bmp.GetHeight()
            hasmask = bmp.GetMask() is not None
        else:
            bmpwidth = bmpheight = 0

        dc.SetFont(self.GetFont())
        color = (self.GetForegroundColour() if self.IsEnabled() else
                 wx.SystemSettings.GetColour(wx.SYS_COLOUR_GRAYTEXT))
        dc.SetTextForeground(color)

        label = self.GetLabel()
        txtwidth, txtheight = dc.GetTextExtent(label)
        # center bitmap and text
        xpos = (width - bmpwidth - txtwidth) // 2 if center else 0
        if bmp is not None:
            dc.DrawBitmap(bmp, xpos, (height - bmpheight) // 2, hasmask)
            xpos += 5
        dc.DrawText(label, xpos + bmpwidth, (height - txtheight) // 2)


class SelectionEvent(wx.PyCommandEvent):
    """Notification of changed element."""

    def __init__(self, evtType, winid, sel):
        wx.PyCommandEvent.__init__(self, evtType, winid)
        self.selection = sel

    def SetSelection(self, select):
        self.selection = select

    def GetSelection(self):
        return self.selection


class WxPeriodicTable(wx.Frame):
    """Main application window."""

    # Class used to instanciate the panel with the elements.
    periodic_panel_class = PeriodicPanel

    def __init__(self, *args, **kwds):
        kwds["style"] = (wx.DEFAULT_DIALOG_STYLE | wx.MINIMIZE_BOX | wx.TAB_TRAVERSAL)
        wx.Frame.__init__(self, *args, **kwds)
        self.selected = -1

        self.SetTitle(MainApp.name)
        icon = wx.EmptyIcon()
        #icon.CopyFromBitmap(wx.Bitmap(MainApp.icon + '.png',
        #                              wx.BITMAP_TYPE_ANY))
        self.SetIcon(icon)
        self.SetBackgroundColour(wx.SystemSettings_GetColour(
            wx.SYS_COLOUR_3DFACE))

        # create menu
        self.menu = wx.MenuBar()
        self.SetMenuBar(self.menu)
        menu = wx.Menu()
        menu.Append(wx.ID_EXIT, "Exit", "Exit the application", wx.ITEM_NORMAL)
        self.menu.Append(menu, "File")
        menu = wx.Menu()
        menu.Append(wx.ID_COPY,
            "Copy\tCtrl+C", "Copy selected element to the clipboard",
            wx.ITEM_NORMAL)
        self.menu.Append(menu, "Edit")
        menu = wx.Menu()
        menu.Append(wx.ID_VIEW_DETAILS, "Details", "Show or hide details",
            wx.ITEM_CHECK)
        self.menu.Append(menu, "View")
        menu = wx.Menu()
        menu.Append(wx.ID_ABOUT, "About...",
            "Display information about the program", wx.ITEM_NORMAL)
        self.menu.Append(menu, "Help")

        # create panels and controls
        self.notebook = wx.Notebook(self, -1, style=0)
        self.description = DecriptionPanel(self.notebook, -1)
        self.details = DetailsPanel(self.notebook, -1)

        self.table = self.periodic_panel_class(self, id=-1)

        self.disclose = DisclosureCtrl(self.table, -1, '')
        self.table.AddCtrl(self.disclose)
        self.notebook.AddPage(self.description, "Description")
        self.notebook.AddPage(self.details, "Properties")

        # event bindings
        self.Bind(wx.EVT_MENU, self.OnExit, id=wx.ID_EXIT)
        self.Bind(wx.EVT_MENU, self.OnCopy, id=wx.ID_COPY)
        self.Bind(wx.EVT_MENU, self.OnDetails, id=wx.ID_VIEW_DETAILS)
        self.Bind(wx.EVT_MENU, self.OnAbout, id=wx.ID_ABOUT)
        self.Bind(EVT_ELE_CHANGED, self.OnSelect, self.table)
        self.Bind(EVT_ELE_CHANGED, self.OnSelect, self.details)
        self.Bind(wx.EVT_BUTTON, self.OnDetails, self.disclose)

        # create sizers
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.table, 1, (wx.LEFT|wx.TOP|wx.RIGHT|wx.EXPAND|
            wx.ALIGN_CENTER_HORIZONTAL| wx.EXPAND|wx.ADJUST_MINSIZE), BORDER-5)
        self.sizer.Add((BORDER, BORDER))
        self.sizer.Add(self.notebook, 0, (wx.LEFT|wx.RIGHT|wx.BOTTOM|
            wx.EXPAND|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL|
            wx.ADJUST_MINSIZE), BORDER)

        self.notebook.SetSelection(1)
        self.SetAutoLayout(True)
        self.SetSizerAndFit(self.sizer, True)
        self.sizer.SetSizeHints(self)
        self.ApplyLayout(True)
        self.SetSelection(0)

    def ApplyLayout(self, show=False):
        self.show_info = show
        self.sizer.Show(2, show, True)
        self.SetAutoLayout(True)
        self.SetSizerAndFit(self.sizer, True)
        self.sizer.SetSizeHints(self)
        self.Layout()
        self.menu.Check(wx.ID_VIEW_DETAILS, show)
        self.disclose.SetToggle(self.show_info)

    def OnDetails(self, evt):
        self.ApplyLayout(not self.show_info)

    def OnEraseBackground(self, event):
        pass

    def OnCopy(self, evt):
        dobj = wx.TextDataObject()
        dobj.SetText(repr(ELEMENTS[self.selected + 1]))
        if wx.TheClipboard.Open():
            wx.TheClipboard.SetData(dobj)
            wx.TheClipboard.Close()

    def OnAbout(self, evt):
        info = wx.AboutDialogInfo()
        info.Name = MainApp.name
        info.Version = MainApp.version
        info.Copyright = MainApp.copyright
        info.WebSite = MainApp.website
        wx.AboutBox(info)

    def OnWebsite(self, evt):
        webbrowser.open(MainApp.website)

    def OnWikipedia(self, evt):
        webbrowser.open("http://en.wikipedia.org/wiki/%s" % (
            ELEMENTS[self.selected].name), 1)

    def OnWebElements(self, evt):
        webbrowser.open("http://www.webelements.com/%s/" % (
            ELEMENTS[self.selected].name.lower()))

    def OnSelect(self, evt):
        self.SetSelection(evt.GetSelection())

    def SetSelection(self, select):
        """Set active element."""
        if self.selected != select:
            self.selected = select
            self.description.SetSelection(select)
            self.table.SetSelection(select)
            self.details.SetSelection(select)

    def OnExit(self, evt):
        self.Close()
        if __name__ == "__main__":
            sys.exit(0)
        else:
            return self.selected


pteEVT_ELE_CHANGED = wx.NewEventType()
EVT_ELE_CHANGED = wx.PyEventBinder(pteEVT_ELE_CHANGED, 1)

SPACER = 12 if wx.Platform == "__WXMAC__" else 10
BORDER = 22 if wx.Platform == "__WXMAC__" else 10

COLORS = {
    1: (0x99, 0xff, 0x99),  # Nonmetals
    2: (0xc0, 0xff, 0xff),  # Noble gases
    3: (0xff, 0x99, 0x99),  # Alkali metals
    4: (0xff, 0xde, 0xad),  # Alkaline earth metals
    5: (0xcc, 0xcc, 0x99),  # Metalloids
    6: (0xff, 0xff, 0x99),  # Halogens
    7: (0xcc, 0xcc, 0xcc),  # Poor metals
    8: (0xff, 0xc0, 0xc0),  # Transition metals
    9: (0xff, 0xbf, 0xff),  # Lanthanides
    10: (0xff, 0x99, 0xcc),  # Actinides
}

_u = (lambda x: x.decode('latin-1')) if sys.version[0] == '2' else str

if __name__ == "__main__":
    MainApp(0).MainLoop()



#!/usr/bin/env python
r"""
Flow for Born effective charges and dielectric tensors with DFPT
================================================================

This example shows how to compute the Born effective charges and
the dielectric tensors (e0, einf) of AlAs with AbiPy flows.
We perform multiple calculations by varying the number of k-points
to analyze the convergence of the results wrt nkpt
"""
import sys
import os
import abipy.abilab as abilab
import abipy.data as abidata

from abipy import flowtk


def make_scf_input(ngkpt, paral_kgb=0):
    """
    This function constructs the input file for the GS calculation for a given IBZ sampling.
    """
    # Crystalline AlAs: computation of the second derivative of the total energy
    structure = abidata.structure_from_ucell("AlAs")
    pseudos = abidata.pseudos("13al.981214.fhi", "33as.pspnc")
    gs_inp = abilab.AbinitInput(structure, pseudos=pseudos)

    gs_inp.set_vars(
        nband=4,
        ecut=2.0,
        ngkpt=ngkpt,
        nshiftk=4,
        shiftk=[0.0, 0.0, 0.5,   # This gives the usual fcc Monkhorst-Pack grid
                0.0, 0.5, 0.0,
                0.5, 0.0, 0.0,
                0.5, 0.5, 0.5],
        #shiftk=[0, 0, 0],
        paral_kgb=paral_kgb,
        tolvrs=1.0e-10,
        ixc=1,
        diemac=9.0,
        #iomode=3,
    )

    return gs_inp


def build_flow(options):
    """
    Create a `Flow` for phonon calculations. The flow has two works.

    The first work contains a single GS task that produces the WFK file used in DFPT
    Then we have multiple Works that are generated automatically
    in order to compute the dynamical matrix on a [4, 4, 4] mesh.
    Symmetries are taken into account: only q-points in the IBZ are generated and
    for each q-point only the independent atomic perturbations are computed.
    """
    # Working directory (default is the name of the script with '.py' removed and "run_" replaced by "flow_")
    if not options.workdir:
        options.workdir = os.path.basename(sys.argv[0]).replace(".py", "").replace("run_", "flow_")

    flow = flowtk.Flow(workdir=options.workdir)

    for ngkpt in [(2, 2, 2), (4, 4, 4), (8, 8, 8)]:
        # Build input for GS calculation
        scf_input = make_scf_input(ngkpt=ngkpt)
        flow.register_scf_task(scf_input, append=True)

    for scf_task in flow[0]:
        bec_work = flowtk.BecWork.from_scf_task(scf_task)
        flow.register_work(bec_work)

    return flow


# This block generates the thumbnails in the AbiPy gallery.
# You can safely REMOVE this part if you are using this script for production runs.
if os.getenv("READTHEDOCS", False):
    __name__ = None
    import tempfile
    options = flowtk.build_flow_main_parser().parse_args(["-w", tempfile.mkdtemp()])
    build_flow(options).graphviz_imshow()


@flowtk.flow_main
def main(options):
    """
    This is our main function that will be invoked by the script.
    flow_main is a decorator implementing the command line interface.
    Command line args are stored in `options`.
    """
    return build_flow(options)


if __name__ == "__main__":
    sys.exit(main())


############################################################################
#
# Run the script with:
#
#     run_becs_and_epsilon_vs_kpts.py -s
#
# Use:
#
#   abirun.py flow_becs_and_epsilon_vs_kpts/ listext DDB
#
# to list all the DDB files produced by the flow.
#
# Now use abicomp.py to create a DdbRobot to analyze the DDB files produced by the 3 Works:
#
#    abicomp.py ddb flow_becs_and_epsilon_vs_kpts/w*/outdata/out_DDB
#
# the inside the ipython terminal use the anacompare methods to analyze the results. e.g.
#
# .. code-block:: ipython
#
#     In [1]: r = robot.anacompare_epsinf()
#     In [2]: r.df
#     Out[2]:
#               xx         yy         zz   yz   xz   xy  formula  chneut  \
#     0  13.584082  13.584082  13.584082  0.0  0.0  0.0  Al1 As1       1
#     0   9.268425   9.268425   9.268425  0.0  0.0  0.0  Al1 As1       1
#     0   8.873178   8.873178   8.873178  0.0  0.0  0.0  Al1 As1       1
#



from __future__ import print_function, division

import abc
import sys
import wx

#from abipy.gui.awx.dialogs import showErrorMessage


__all__ = [
    "App",
]

def ExceptionHook(exctype, value, trace):
    """
    Handler for all unhandled exceptions
    Create a simple exception hook to handle and inform the user of
    any unexpected errors that occur while the program is running:

    @param exctype: Exception Type
    @param value: Error Value
    @param trace: Trace back info
    """
    # Format the traceback
    import traceback
    exc = traceback.format_exception(exctype, value, trace)
    ftrace = "".join(exc)
    app = wx.GetApp()

    if app:
        msg = "An unexpected error has occurred: %s" % ftrace
        parent = app.GetTopWindow()
        parent.Raise()
        wx.MessageBox(parent=parent, message=msg, caption=app.GetAppName(), style=wx.ICON_ERROR|wx.OK)
        #showErrorMessage(parent=parent, message=None)
        #app.Exit()
    else:
        sys.stderr.write(ftrace)


class App(wx.App):
    __metaclass__ = abc.ABCMeta

    def __init__(self, *args, **kwargs):
        wx.App.__init__(self, *args, **kwargs)

        # Initialize the logger
        import logging
        loglevel = "WARNING"
        loglevel = "DEBUG"
        loglevel = "CRITICAL"

        numeric_level = getattr(logging, loglevel.upper(), None)
        if not isinstance(numeric_level, int):
            raise ValueError('Invalid log level: %s' % loglevel)
        logging.basicConfig(level=numeric_level)

        # This catches events when the app is asked to activate by some other process
        self.Bind(wx.EVT_ACTIVATE_APP, self.OnActivate)

    def OnInit(self):
        # Handler for all unhandled exceptions
        sys.excepthook = ExceptionHook

        # Enforce WXAgg as matplotlib backend to avoid nasty SIGSEGV in the C++ layer
        # occurring when WX Guis produce plots with other backends.
        import matplotlib
        matplotlib.use('WXAgg')
        return True

    @property
    def appname(self):
        """Name of the application."""
        return self.__class__.__name__

    def __repr__(self):
        return "<%s at %s>" % (self.appname, id(self))

    def __str__(self):
        return self.__repr__()

    def BringWindowToFront(self):
        try:
            # it's possible for this event to come when the frame is closed
            self.GetTopWindow().Raise()
        except:
            pass

    def OnActivate(self, event):
        # if this is an activate event, rather than something else, like iconize.
        if event.GetActive():
            self.BringWindowToFront()
        event.Skip()

    #@abc.abstractmethod
    #def MacOpenFile(self, filename):
    #    """Called for files droped on dock icon, or opened via finders context menu"""
    #    #if filename.endswith(".py"):
    #    #    return
    #    # Code to load filename.
    #    #self.log("%s dropped on app %s" % (filename, self.appname))
    #
    #def MacReopenApp(self):
    #    """Called when the dock icon is clicked."""
    #    self.BringWindowToFront()

    #def MacNewFile(self):
    #    pass

    #def MacPrintFile(self, filepath):
    #    pass



# coding: utf-8
"""
Utilities for generating matplotlib plots.

.. note::

    Avoid importing matplotlib in the module namespace otherwise startup is very slow.
"""
import os
import time
import itertools
import numpy as np

from collections import OrderedDict, namedtuple
from pymatgen.util.plotting import add_fig_kwargs, get_ax_fig_plt, get_ax3d_fig_plt, get_axarray_fig_plt
from .numtools import data_from_cplx_mode


__all__ = [
    "set_axlims",
    "get_ax_fig_plt",
    "get_ax3d_fig_plt",
    "plot_array",
    "ArrayPlotter",
    "data_from_cplx_mode",
    "Marker",
    "plot_unit_cell",
    "GenericDataFilePlotter",
    "GenericDataFilesPlotter",
]


# https://matplotlib.org/gallery/lines_bars_and_markers/linestyles.html
linestyles = OrderedDict(
    [('solid',               (0, ())),
     ('loosely_dotted',      (0, (1, 10))),
     ('dotted',              (0, (1, 5))),
     ('densely_dotted',      (0, (1, 1))),

     ('loosely_dashed',      (0, (5, 10))),
     ('dashed',              (0, (5, 5))),
     ('densely_dashed',      (0, (5, 1))),

     ('loosely_dashdotted',  (0, (3, 10, 1, 10))),
     ('dashdotted',          (0, (3, 5, 1, 5))),
     ('densely_dashdotted',  (0, (3, 1, 1, 1))),

     ('loosely_dashdotdotted', (0, (3, 10, 1, 10, 1, 10))),
     ('dashdotdotted',         (0, (3, 5, 1, 5, 1, 5))),
     ('densely_dashdotdotted', (0, (3, 1, 1, 1, 1, 1)))]
)


def ax_append_title(ax, title, loc="center", fontsize=None):
    """Add title to previous ax.title. Return new title."""
    prev_title = ax.get_title(loc=loc)
    new_title = prev_title + title
    ax.set_title(new_title, loc=loc, fontsize=fontsize)
    return new_title


def ax_share(xy_string, *ax_list):
    """
    Share x- or y-axis of two or more subplots after they are created

    Args:
        xy_string: "x" to share x-axis, "xy" for both
        ax_list: List of axes to share.

    Example:

        ax_share("y", ax0, ax1)
        ax_share("xy", *(ax0, ax1, ax2))
    """
    if "x" in xy_string:
        for ix, ax in enumerate(ax_list):
            others = [a for a in ax_list if a != ax]
            ax.get_shared_x_axes().join(*others)

    if "y" in xy_string:
        for ix, ax in enumerate(ax_list):
            others = [a for a in ax_list if a != ax]
            ax.get_shared_y_axes().join(*others)


#def set_grid(fig, boolean):
#    if hasattr(fig, "axes"):
#        for ax in fig.axes:
#            if ax.grid: ax.grid.set_visible(boolean)
#    else:
#            if ax.grid: ax.grid.set_visible(boolean)


def set_axlims(ax, lims, axname):
    """
    Set the data limits for the axis ax.

    Args:
        lims: tuple(2) for (left, right), tuple(1) or scalar for left only.
        axname: "x" for x-axis, "y" for y-axis.

    Return: (left, right)
    """
    left, right = None, None
    if lims is None: return (left, right)

    len_lims = None
    try:
        len_lims = len(lims)
    except TypeError:
        # Assume Scalar
        left = float(lims)

    if len_lims is not None:
        if len(lims) == 2:
            left, right = lims[0], lims[1]
        elif len(lims) == 1:
            left = lims[0]

    set_lim = getattr(ax, {"x": "set_xlim", "y": "set_ylim"}[axname])
    if left != right:
        set_lim(left, right)

    return left, right


def set_ax_xylabels(ax, xlabel, ylabel, exchange_xy):
    """
    Set the x- and the y-label of axis ax, exchanging x and y if exchange_xy
    """
    if exchange_xy: xlabel, ylabel = ylabel, xlabel
    ax.set_xlabel(xlabel)
    ax.set_ylabel(ylabel)


def set_visible(ax, boolean, *args):
    """
    Hide/Show the artists of axis ax listed in args.
    """
    if "legend" in args and ax.legend():
        ax.legend().set_visible(boolean)
    if "title" in args and ax.title:
        ax.title.set_visible(boolean)
    if "xlabel" in args and ax.xaxis.label:
        ax.xaxis.label.set_visible(boolean)
    if "ylabel" in args and ax.yaxis.label:
        ax.yaxis.label.set_visible(boolean)
    if "xticklabels" in args:
        for label in ax.get_xticklabels():
            label.set_visible(boolean)
    if "yticklabels" in args:
        for label in ax.get_yticklabels():
            label.set_visible(boolean)


def rotate_ticklabels(ax, rotation, axname="x"):
    """Rotate the ticklables of axis ``ax``"""
    if "x" in axname:
        for tick in ax.get_xticklabels():
            tick.set_rotation(rotation)
    if "y" in axname:
        for tick in ax.get_yticklabels():
            tick.set_rotation(rotation)


@add_fig_kwargs
def plot_xy_with_hue(data, x, y, hue, decimals=None, ax=None,
                     xlims=None, ylims=None, fontsize=12, **kwargs):
    """
    Plot y = f(x) relation for different values of `hue`.
    Useful for convergence tests done wrt to two parameters.

    Args:
        data: |pandas-DataFrame| containing columns `x`, `y`, and `hue`.
        x: Name of the column used as x-value
        y: Name of the column(s) used as y-value
        hue: Variable that define subsets of the data, which will be drawn on separate lines
        decimals: Number of decimal places to round `hue` columns. Ignore if None
        ax: |matplotlib-Axes| or None if a new figure should be created.
        xlims ylims: Set the data limits for the x(y)-axis. Accept tuple e.g. `(left, right)`
            or scalar e.g. `left`. If left (right) is None, default values are used
        fontsize: Legend fontsize.
        kwargs: Keywork arguments are passed to ax.plot method.

    Returns: |matplotlib-Figure|
    """
    if isinstance(y, (list, tuple)):
        # Recursive call for each ax in ax_list.
        num_plots, ncols, nrows = len(y), 1, 1
        if num_plots > 1:
            ncols = 2
            nrows = (num_plots // ncols) + (num_plots % ncols)

        ax_list, fig, plt = get_axarray_fig_plt(None, nrows=nrows, ncols=ncols,
                                                sharex=False, sharey=False, squeeze=False)

        ax_list = ax_list.ravel()
        if num_plots % ncols != 0: ax_list[-1].axis('off')

        for yname, ax in zip(y, ax_list):
            plot_xy_with_hue(data, x, str(yname), hue, decimals=decimals, ax=ax,
                             xlims=xlims, ylims=ylims, fontsize=fontsize, show=False, **kwargs)
        return fig

    # Check here because pandas error messages are a bit criptic.
    miss = [k for k in (x, y, hue) if k not in data]
    if miss:
        raise ValueError("Cannot find `%s` in dataframe.\nAvailable keys are: %s" % (str(miss), str(data.keys())))

    # Truncate values in hue column so that we can group.
    if decimals is not None:
        data = data.round({hue: decimals})

    ax, fig, plt = get_ax_fig_plt(ax=ax)
    for key, grp in data.groupby(hue):
        # Sort xs and rearrange ys
        xy = np.array(sorted(zip(grp[x], grp[y]), key=lambda t: t[0]))
        xvals, yvals = xy[:, 0], xy[:, 1]

        #label = "{} = {}".format(hue, key)
        label = "%s" % (str(key))
        if not kwargs:
            ax.plot(xvals, yvals, 'o-', label=label)
        else:
            ax.plot(xvals, yvals, label=label, **kwargs)

    ax.grid(True)
    ax.set_xlabel(x)
    ax.set_ylabel(y)
    set_axlims(ax, xlims, "x")
    set_axlims(ax, ylims, "y")
    ax.legend(loc="best", fontsize=fontsize, shadow=True)

    return fig


@add_fig_kwargs
def plot_array(array, color_map=None, cplx_mode="abs", **kwargs):
    """
    Use imshow for plotting 2D or 1D arrays.

    Example::

        plot_array(np.random.rand(10,10))

    See <http://stackoverflow.com/questions/7229971/2d-grid-data-visualization-in-python>

    Args:
        array: Array-like object (1D or 2D).
        color_map: color map.
        cplx_mode:
            Flag defining how to handle complex arrays. Possible values in ("re", "im", "abs", "angle")
            "re" for the real part, "im" for the imaginary part.
            "abs" means that the absolute value of the complex number is shown.
            "angle" will display the phase of the complex number in radians.

    Returns: |matplotlib-Figure|
    """
    # Handle vectors
    array = np.atleast_2d(array)
    array = data_from_cplx_mode(cplx_mode, array)

    import matplotlib as mpl
    from matplotlib import pyplot as plt
    if color_map is None:
        # make a color map of fixed colors
        color_map = mpl.colors.LinearSegmentedColormap.from_list('my_colormap',
                                                                 ['blue', 'black', 'red'], 256)

    img = plt.imshow(array, interpolation='nearest', cmap=color_map, origin='lower')

    # Make a color bar
    plt.colorbar(img, cmap=color_map)

    # Set grid
    plt.grid(True, color='white')

    fig = plt.gcf()
    return fig


class ArrayPlotter(object):

    def __init__(self, *labels_and_arrays):
        """
        Args:
            labels_and_arrays: List [("label1", arr1), ("label2", arr2")]
        """
        self._arr_dict = OrderedDict()
        for label, array in labels_and_arrays:
            self.add_array(label, array)

    def __len__(self):
        return len(self._arr_dict)

    def __iter__(self):
        return self._arr_dict.__iter__()

    def keys(self):
        return self._arr_dict.keys()

    def items(self):
        return self._arr_dict.items()

    def add_array(self, label, array):
        """Add array with the given name."""
        if label in self._arr_dict:
            raise ValueError("%s is already in %s" % (label, list(self._arr_dict.keys())))

        self._arr_dict[label] = array

    def add_arrays(self, labels, arr_list):
        """
        Add a list of arrays

        Args:
            labels: List of labels.
            arr_list: List of arrays.
        """
        assert len(labels) == len(arr_list)
        for label, arr in zip(labels, arr_list):
            self.add_array(label, arr)

    @add_fig_kwargs
    def plot(self, cplx_mode="abs", colormap="jet", fontsize=8, **kwargs):
        """
        Args:
            cplx_mode: "abs" for absolute value, "re", "im", "angle"
            colormap: matplotlib colormap.
            fontsize: legend and label fontsize.

        Returns: |matplotlib-Figure|
        """
        # Build grid of plots.
        num_plots, ncols, nrows = len(self), 1, 1
        if num_plots > 1:
            ncols = 2
            nrows = num_plots // ncols + (num_plots % ncols)

        import matplotlib.pyplot as plt
        fig, ax_mat = plt.subplots(nrows=nrows, ncols=ncols, sharex=False, sharey=False, squeeze=False)
        # Don't show the last ax if num_plots is odd.
        if num_plots % ncols != 0: ax_mat[-1, -1].axis("off")

        from mpl_toolkits.axes_grid1 import make_axes_locatable
        from matplotlib.ticker import MultipleLocator

        for ax, (label, arr) in zip(ax_mat.flat, self.items()):
            data = data_from_cplx_mode(cplx_mode, arr)
            # Use origin to place the [0, 0] index of the array in the lower left corner of the axes.
            img = ax.matshow(data, interpolation='nearest', cmap=colormap, origin='lower', aspect="auto")
            ax.set_title("(%s) %s" % (cplx_mode, label), fontsize=fontsize)

            # Make a color bar for this ax
            # Create divider for existing axes instance
            # http://stackoverflow.com/questions/18266642/multiple-imshow-subplots-each-with-colorbar
            divider3 = make_axes_locatable(ax)
            # Append axes to the right of ax, with 10% width of ax
            cax3 = divider3.append_axes("right", size="10%", pad=0.05)
            # Create colorbar in the appended axes
            # Tick locations can be set with the kwarg `ticks`
            # and the format of the ticklabels with kwarg `format`
            cbar3 = plt.colorbar(img, cax=cax3, ticks=MultipleLocator(0.2), format="%.2f")
            # Remove xticks from ax
            ax.xaxis.set_visible(False)
            # Manually set ticklocations
            #ax.set_yticks([0.0, 2.5, 3.14, 4.0, 5.2, 7.0])

            # Set grid
            ax.grid(True, color='white')

        fig.tight_layout()
        return fig


#TODO use object and introduce c for color, client code should be able to customize it.
# Rename it to ScatterData
class Marker(namedtuple("Marker", "x y s")):
    """
    Stores the position and the size of the marker.
    A marker is a list of tuple(x, y, s) where x, and y are the position
    in the graph and s is the size of the marker.
    Used for plotting purpose e.g. QP data, energy derivatives...

    Example::

        x, y, s = [1, 2, 3], [4, 5, 6], [0.1, 0.2, -0.3]
        marker = Marker(x, y, s)
        marker.extend((x, y, s))

    """
    def __new__(cls, *xys):
        """Extends the base class adding consistency check."""
        if not xys:
            xys = ([], [], [])
            return super().__new__(cls, *xys)

        if len(xys) != 3:
            raise TypeError("Expecting 3 entries in xys got %d" % len(xys))

        x = np.asarray(xys[0])
        y = np.asarray(xys[1])
        s = np.asarray(xys[2])
        xys = (x, y, s)

        for s in xys[-1]:
            if np.iscomplex(s):
                raise ValueError("Found ambiguous complex entry %s" % str(s))

        return super().__new__(cls, *xys)

    def __bool__(self):
        return bool(len(self.s))

    __nonzero__ = __bool__

    def extend(self, xys):
        """
        Extend the marker values.
        """
        if len(xys) != 3:
            raise TypeError("Expecting 3 entries in xys got %d" % len(xys))

        self.x.extend(xys[0])
        self.y.extend(xys[1])
        self.s.extend(xys[2])

        lens = np.array((len(self.x), len(self.y), len(self.s)))
        if np.any(lens != lens[0]):
            raise TypeError("x, y, s vectors should have same lengths but got %s" % str(lens))

    def posneg_marker(self):
        """
        Split data into two sets: the first one contains all the points with positive size.
        The first set contains all the points with negative size.
        """
        pos_x, pos_y, pos_s = [], [], []
        neg_x, neg_y, neg_s = [], [], []

        for x, y, s in zip(self.x, self.y, self.s):
            if s >= 0.0:
                pos_x.append(x)
                pos_y.append(y)
                pos_s.append(s)
            else:
                neg_x.append(x)
                neg_y.append(y)
                neg_s.append(s)

        return self.__class__(pos_x, pos_y, pos_s), Marker(neg_x, neg_y, neg_s)


class MplExpose(object): # pragma: no cover
    """
    Example:

        with MplExpose() as e:
            e(obj.plot1(show=False))
            e(obj.plot2(show=False))
    """
    def __init__(self, slide_mode=False, slide_timeout=None, verbose=1):
        """
        Args:
            slide_mode: If true, iterate over figures. Default: Expose all figures at once.
            slide_timeout: Close figure after slide-timeout seconds Block if None.
            verbose: verbosity level
        """
        self.figures = []
        self.slide_mode = bool(slide_mode)
        self.timeout_ms = slide_timeout
        self.verbose = verbose
        if self.timeout_ms is not None:
            self.timeout_ms = int(self.timeout_ms * 1000)
            assert self.timeout_ms >= 0

        if self.verbose:
            if self.slide_mode:
                print("\nSliding matplotlib figures with slide timeout: %s [s]" % slide_timeout)
            else:
                print("\nLoading all matplotlib figures before showing them. It may take some time...")

        self.start_time = time.time()

    def __call__(self, obj):
        """
        Add an object to MplExpose. Support mpl figure, list of figures or
        generator yielding figures.
        """
        import types
        if isinstance(obj, (types.GeneratorType, list, tuple)):
            for fig in obj:
                self.add_fig(fig)
        else:
            self.add_fig(obj)

    def add_fig(self, fig):
        """Add a matplotlib figure."""
        if fig is None: return

        if not self.slide_mode:
            self.figures.append(fig)
        else:
            #print("Printing and closing", fig)
            import matplotlib.pyplot as plt
            if self.timeout_ms is not None:
                # Creating a timer object
                # timer calls plt.close after interval milliseconds to close the window.
                timer = fig.canvas.new_timer(interval=self.timeout_ms)
                timer.add_callback(plt.close, fig)
                timer.start()

            plt.show()
            fig.clear()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Activated at the end of the with statement. """
        self.expose()

    def expose(self):
        """Show all figures. Clear figures if needed."""
        if not self.slide_mode:
            print("All figures in memory, elapsed time: %.3f s" % (time.time() - self.start_time))
            import matplotlib.pyplot as plt
            plt.show()
            for fig in self.figures:
                fig.clear()


def plot_unit_cell(lattice, ax=None, **kwargs):
    """
    Adds the unit cell of the lattice to a matplotlib Axes3D

    Args:
        lattice: Lattice object
        ax: matplotlib :class:`Axes3D` or None if a new figure should be created.
        kwargs: kwargs passed to the matplotlib function 'plot'. Color defaults to black
            and linewidth to 3.

    Returns:
        matplotlib figure and ax
    """
    ax, fig, plt = get_ax3d_fig_plt(ax)

    if "color" not in kwargs: kwargs["color"] = "k"
    if "linewidth" not in kwargs: kwargs["linewidth"] = 3

    v = 8 * [None]
    v[0] = lattice.get_cartesian_coords([0.0, 0.0, 0.0])
    v[1] = lattice.get_cartesian_coords([1.0, 0.0, 0.0])
    v[2] = lattice.get_cartesian_coords([1.0, 1.0, 0.0])
    v[3] = lattice.get_cartesian_coords([0.0, 1.0, 0.0])
    v[4] = lattice.get_cartesian_coords([0.0, 1.0, 1.0])
    v[5] = lattice.get_cartesian_coords([1.0, 1.0, 1.0])
    v[6] = lattice.get_cartesian_coords([1.0, 0.0, 1.0])
    v[7] = lattice.get_cartesian_coords([0.0, 0.0, 1.0])

    for i, j in ((0, 1), (1, 2), (2, 3), (0, 3), (3, 4), (4, 5), (5, 6),
                 (6, 7), (7, 4), (0, 7), (1, 6), (2, 5), (3, 4)):
        ax.plot(*zip(v[i], v[j]), **kwargs)

    # Plot cartesian frame
    ax_add_cartesian_frame(ax)

    return fig, ax


def ax_add_cartesian_frame(ax, start=(0, 0, 0)):
    """
    Add cartesian frame to 3d axis at point `start`.
    """
    # https://stackoverflow.com/questions/22867620/putting-arrowheads-on-vectors-in-matplotlibs-3d-plot
    from matplotlib.patches import FancyArrowPatch
    from mpl_toolkits.mplot3d import proj3d
    arrow_opts = {"color": "k"}
    arrow_opts.update(dict(lw=1, arrowstyle="-|>",))

    class Arrow3D(FancyArrowPatch):
        def __init__(self, xs, ys, zs, *args, **kwargs):
            FancyArrowPatch.__init__(self, (0, 0), (0, 0), *args, **kwargs)
            self._verts3d = xs, ys, zs

        def draw(self, renderer):
            xs3d, ys3d, zs3d = self._verts3d
            xs, ys, zs = proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
            self.set_positions((xs[0], ys[0]), (xs[1], ys[1]))
            FancyArrowPatch.draw(self, renderer)

    start = np.array(start)
    for end in ((1, 0, 0), (0, 1, 0), (0, 0, 1)):
        end = start + np.array(end)
        xs, ys, zs = list(zip(start, end))
        p = Arrow3D(xs, ys, zs,
                   connectionstyle='arc3', mutation_scale=20,
                   alpha=0.8, **arrow_opts)
        ax.add_artist(p)

    return ax


def plot_structure(structure, ax=None, to_unit_cell=False, alpha=0.7,
                   style="points+labels", color_scheme="VESTA", **kwargs):
    """
    Plot structure with matplotlib (minimalistic version)

    Args:
        structure: Structure object
        ax: matplotlib :class:`Axes3D` or None if a new figure should be created.
        alpha: The alpha blending value, between 0 (transparent) and 1 (opaque)
        to_unit_cell: True if sites should be wrapped into the first unit cell.
        style: "points+labels" to show atoms sites with labels.
        color_scheme: color scheme for atom types. Allowed values in ("Jmol", "VESTA")

    Returns: |matplotlib-Figure|
    """
    fig, ax = plot_unit_cell(structure.lattice, ax=ax, linewidth=1)

    from pymatgen.analysis.molecule_structure_comparator import CovalentRadius
    from pymatgen.vis.structure_vtk import EL_COLORS
    xyzs, colors = np.empty((len(structure), 4)), []

    for i, site in enumerate(structure):
        symbol = site.specie.symbol
        color = tuple(i / 255 for i in EL_COLORS[color_scheme][symbol])
        radius = CovalentRadius.radius[symbol]
        if to_unit_cell and hasattr(site, "to_unit_cell"): site = site.to_unit_cell()
        # Use cartesian coordinates.
        x, y, z = site.coords
        xyzs[i] = (x, y, z, radius)
        colors.append(color)
        if "labels" in style:
            ax.text(x, y, z, symbol)

    # The definition of sizes is not optimal because matplotlib uses points
    # wherease we would like something that depends on the radius (5000 seems to give reasonable plots)
    # For possibile approaches, see
    # https://stackoverflow.com/questions/9081553/python-scatter-plot-size-and-style-of-the-marker/24567352#24567352
    # https://gist.github.com/syrte/592a062c562cd2a98a83
    if "points" in style:
        x, y, z, s = xyzs.T.copy()
        s = 5000 * s ** 2
        ax.scatter(x, y, zs=z, s=s, c=colors, alpha=alpha)  #facecolors="white", #edgecolors="blue"

    ax.set_title(structure.composition.formula)
    ax.set_axis_off()

    return fig


def _generic_parser_fh(fh):
    """
    Parse file with data in tabular format. Supports multi datasets a la gnuplot.
    Mainly used for files without any schema, not even CSV

    Args:
        fh: File object

    Returns:
        OrderedDict title --> numpy array
        where title is taken from the first (non-empty) line preceding the dataset
    """
    arr_list = [None]
    data = []
    head_list = []
    count = -1
    last_header = None
    for l in fh:
        l = l.strip()
        if not l or l.startswith("#"):
            count = -1
            last_header = l
            if arr_list[-1] is not None: arr_list.append(None)
            continue

        count += 1
        if count == 0: head_list.append(last_header)
        if arr_list[-1] is None: arr_list[-1] = []
        data = arr_list[-1]
        data.append(list(map(float, l.split())))

    if len(head_list) != len(arr_list):
        raise RuntimeError("len(head_list) != len(arr_list), %d != %d" % (len(head_list), len(arr_list)))

    od = OrderedDict()
    for key, data in zip(head_list, arr_list):
        key = " ".join(key.split())
        if key in od:
            print("Header %s already in dictionary. Using new key %s" % (key, 2 * key))
            key = 2 * key
        od[key] = np.array(data).T.copy()

    return od


class GenericDataFilePlotter(object):
    """
    Extract data from a generic text file with results
    in tabular format and plot data with matplotlib.
    Multiple datasets are supported.
    No attempt is made to handle metadata (e.g. column name)
    Mainly used to handle text files written without any schema.
    """
    def __init__(self, filepath):
        with open(filepath, "rt") as fh:
            self.od = _generic_parser_fh(fh)

    def __str__(self):
        return self.to_string()

    def to_string(self, verbose=0):
        """String representation with verbosity level `verbose`."""
        lines = []
        for key, arr in self.od.items():
            lines.append("key: `%s` --> array shape: %s" % (key, str(arr.shape)))
        return "\n".join(lines)

    @add_fig_kwargs
    def plot(self, use_index=False, fontsize=8, **kwargs):
        """
        Plot all arrays. Use multiple axes if datasets.

        Args:
            use_index: By default, the x-values are taken from the first column.
                If use_index is False, the x-values are the row index.
            fontsize: fontsize for title.
            kwargs: options passed to ``ax.plot``.

        Return: |matplotlib-figure|
        """
        # build grid of plots.
        num_plots, ncols, nrows = len(self.od), 1, 1
        if num_plots > 1:
            ncols = 2
            nrows = (num_plots // ncols) + (num_plots % ncols)

        ax_list, fig, plt = get_axarray_fig_plt(None, nrows=nrows, ncols=ncols,
                                                sharex=False, sharey=False, squeeze=False)
        ax_list = ax_list.ravel()

        # Don't show the last ax if num_plots is odd.
        if num_plots % ncols != 0: ax_list[-1].axis("off")

        for ax, (key, arr) in zip(ax_list, self.od.items()):
            ax.set_title(key, fontsize=fontsize)
            ax.grid(True)
            xs = arr[0] if not use_index else list(range(len(arr[0])))
            for ys in arr[1:] if not use_index else arr:
                ax.plot(xs, ys)

        return fig


class GenericDataFilesPlotter(object):

    @classmethod
    def from_files(cls, filepaths):
        """
        Build object from a list of `filenames`.
        """
        new = cls()
        for filepath in filepaths:
            new.add_file(filepath)
        return new

    def __init__(self):
        self.odlist = []
        self.filepaths = []

    def __str__(self):
        return self.to_string()

    def to_string(self, verbose=0):
        lines = []
        app = lines.append
        for od, filepath in zip(self.odlist, self.filepaths):
            app("File: %s" % filepath)
            for key, arr in od.items():
                lines.append("\tkey: `%s` --> array shape: %s" % (key, str(arr.shape)))

        return "\n".join(lines)

    def add_file(self, filepath):
        """Add data from `filepath`"""
        with open(filepath, "rt") as fh:
            self.odlist.append(_generic_parser_fh(fh))
            self.filepaths.append(filepath)

    @add_fig_kwargs
    def plot(self, use_index=False, fontsize=8, colormap="viridis", **kwargs):
        """
        Plot all arrays. Use multiple axes if datasets.

        Args:
            use_index: By default, the x-values are taken from the first column.
                If use_index is False, the x-values are the row index.
            fontsize: fontsize for title.
            colormap: matplotlib color map.
            kwargs: options passed to ``ax.plot``.

        Return: |matplotlib-figure|
        """
        if not self.odlist: return None

        # Compute intersection of all keys.
        # Here we loose the initial ordering in the dict but oh well!
        klist = [list(d.keys()) for d in self.odlist]
        keys = set(klist[0]).intersection(*klist)
        if not keys:
            print("Warning: cannot find common keys in files. Check input data")
            return None

        # Build grid of plots.
        num_plots, ncols, nrows = len(keys), 1, 1
        if num_plots > 1:
            ncols = 2
            nrows = (num_plots // ncols) + (num_plots % ncols)

        ax_list, fig, plt = get_axarray_fig_plt(None, nrows=nrows, ncols=ncols,
                                                sharex=False, sharey=False, squeeze=False)
        ax_list = ax_list.ravel()

        # Don't show the last ax if num_plots is odd.
        if num_plots % ncols != 0: ax_list[-1].axis("off")

        cmap = plt.get_cmap(colormap)
        line_cycle = itertools.cycle(["-", ":", "--", "-.",])

        # One ax for key, each ax may show multiple arrays
        # so we need different line styles that are consistent with input data.
        # Figure may be crowded but it's difficult to do better without metadata
        # so I'm not gonna spend time to implement more complicated logic.
        for ax, key in zip(ax_list, keys):
            ax.set_title(key, fontsize=fontsize)
            ax.grid(True)
            for iod, (od, filepath) in enumerate(zip(self.odlist, self.filepaths)):
                if key not in od: continue
                arr = od[key]
                color = cmap(iod / len(self.odlist))
                xvals = arr[0] if not use_index else list(range(len(arr[0])))
                arr_list = arr[1:] if not use_index else arr
                for iarr, (ys, linestyle) in enumerate(zip(arr_list, line_cycle)):
                    ax.plot(xvals, ys, color=color, linestyle=linestyle,
                            label=os.path.relpath(filepath) if iarr == 0 else None)

            ax.legend(loc="best", fontsize=fontsize, shadow=True)

        return fig



import os
import wx
import abipy.gui.awx as awx

from wx.py.shell import Shell
from monty.string import marquee
from abipy.abilab import abiopen
from abipy.iotools.visualizer import Visualizer
from abipy.gui import mixins as mix
from abipy.gui.kpoints import SpinKpointBandPanel
from abipy.gui.baseviewer import MultiViewerFrame


class WfkViewerFrame(MultiViewerFrame, mix.Has_Structure, mix.Has_MultipleEbands, mix.Has_Tools, mix.Has_NetcdfFiles):
    VERSION = "0.1"

    HELP_MSG = """Quick help:

 Kpoint list:

     Left-Click:   to visualize u(r)^2 for the selected spin, k-point, band
     Right-Click:  display popup menu with choices.

Also, these key bindings can be used
(For Mac OSX, replace 'Ctrl' with 'Apple'):

  Ctrl-Q:     quit
"""
    @property
    def codename(self):
        return "WfkViewer"

    @property
    def active_wfk(self):
        """The active WFK file i.e. the WFK associated to the active tab."""
        return self.active_tab.wfk

    @property
    def structure(self):
        """`Structure` associated to the active tab."""
        return self.active_wfk.structure

    @property
    def ebands(self):
        """`ElectronBands` associated to the active tab."""
        return self.active_wfk.ebands

    @property
    def ebands_list(self):
        """List of `ElectronBands`."""
        ebands_list = []
        for page in range(self.notebook.GetPageCount()):
            tab = self.notebook.GetPage(page)
            ebands_list.append(tab.wfk.ebands)
        return ebands_list

    @property
    def ebands_filepaths(self):
        """
        Return a list with the absolute paths of the files
        from which the `ElectronBands` have been read.
        """
        paths = []
        for page in range(self.notebook.GetPageCount()):
            tab = self.notebook.GetPage(page)
            paths.append(tab.wfk.filepath)
        return paths

    @property
    def nc_filepaths(self):
        """String with the absolute path of the active netcdf file."""
        paths = []
        for page in range(self.notebook.GetPageCount()):
            tab = self.notebook.GetPage(page)
            paths.append(tab.wfk.filepath)
        return paths

    def makeMenu(self):
        """Creates the main menu."""
        # Base menu.
        menu_bar = super(WfkViewerFrame, self).makeMenu()

        # Add Mixin menus.
        menu_bar.Append(self.CreateStructureMenu(), "Structure")
        menu_bar.Append(self.CreateEbandsMenu(), "Ebands")
        menu_bar.Append(self.CreateToolsMenu(), "Tools")
        menu_bar.Append(self.CreateNetcdfMenu(), "Netcdf")

        # Help menu
        help_menu = self.makeHelpMenu()
        menu_bar.Append(help_menu, "Help")

        #self.menu_bar = menu_bar
        self.SetMenuBar(menu_bar)

    def makeToolBar(self):
        """Creates the toolbar."""
        self.toolbar = toolbar = self.CreateToolBar()
        toolbar.SetToolBitmapSize(wx.Size(48, 48))

        def bitmap(path):
            return wx.Bitmap(awx.path_img(path))

        artBmp = wx.ArtProvider.GetBitmap
        toolbar.AddSimpleTool(wx.ID_OPEN, artBmp(wx.ART_FILE_OPEN, wx.ART_TOOLBAR), "Open")
        toolbar.AddSeparator()

        # Combo box with the list of visualizers
        avail_visunames = [visu.name for visu in Visualizer.get_available()]
        value = avail_visunames[0] if avail_visunames else "None"
        self.visualizer_cbox = wx.ComboBox(toolbar, id=-1, name='visualizer', choices=avail_visunames, value=value, style=wx.CB_READONLY)
        toolbar.AddControl(control=self.visualizer_cbox)

        toolbar.Realize()

    def addFileTab(self, parent, filepath):
        wfk = abiopen(filepath)
        tab = WfkFileTab(self.notebook, wfk)
        self.notebook.AddPage(tab, os.path.basename(filepath))

    def GetVisualizer(self):
        """Returns a string with the visualizer selected by the user."""
        return self.visualizer_cbox.GetValue()


class WfkFileTab(awx.Panel):
    """Tab showing information on a single WFK file."""
    def __init__(self, parent, wfk, **kwargs):
        """
        Args:
            parent:
                parent window.
            wfk:
        """
        super(WfkFileTab, self).__init__(parent, -1, **kwargs)
        self.wfk = wfk

        splitter = wx.SplitterWindow(self, id=-1, style=wx.SP_3D)
        splitter.SetSashGravity(0.95)

        self.skb_panel = skb_panel = SpinKpointBandPanel(splitter, wfk.structure, wfk.nsppol, wfk.kpoints, wfk.mband)

        # Set the callback for double click on k-point row..
        self.Bind(skb_panel.MYEVT_SKB_ACTIVATED, self.onVisualizeSKB)

        # Add Python shell
        msg = "WFK_File object is accessible via the wfk variable. Use wfk.<TAB> to access the list of methods."
        msg = marquee(msg, width=len(msg) + 8, mark="#")
        msg = "#"*len(msg) + "\n" + msg + "\n" + "#"*len(msg) + "\n"

        # FIXME <Error>: CGContextRestoreGState: invalid context 0x0
        pyshell = Shell(splitter, introText=msg,locals={"wfk": self.wfk})
        splitter.SplitHorizontally(self.skb_panel, pyshell)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(splitter, 1, wx.EXPAND, 5)
        self.SetSizerAndFit(sizer)

    @property
    def statusbar(self):
        return self.viewer_frame.statusbar

    def GetVisualizer(self):
        """Returns a string with the visualizer selected by the user."""
        return self.viewer_frame.GetVisualizer()

    def onVisualizeSKB(self, event):
        """
        Calls the visualizer to visualize the specified wavefunction.
        Use the approach described in http://wiki.wxpython.org/LongRunningTasks
        to make the Gui responsive one can use
        """
        #print("in new" ,event.skb)
        spin, kpoint, band = event.skb

        appname = self.GetVisualizer()
        if appname == "None": return

        self.statusbar.PushStatusText("Visualizing wavefunction (spin=%d, kpoint=%s, band=%d)" % (spin, kpoint, band))
        try:
            visu = self.wfk.visualize_ur2(spin, kpoint, band, appname=appname)
            thread = awx.WorkerThread(self, target=visu)
            thread.start()

        except:
            awx.showErrorMessage(self)

    #def _visualize_skb(self, spin, kpoint, band):
    #    """Calls the visualizer to visualize the specified wavefunction."""
    #    # To make the Gui responsive one can use the approach described in
    #    # http://wiki.wxpython.org/LongRunningTasks
    #    appname = self.GetVisualizer()
    #    if appname == "None": return
    #
    #    self.statusbar.PushStatusText("Visualizing wavefunction (spin=%d, kpoint=%s, band=%d)" % (spin, kpoint, band))
    #    try:
    #        visu = self.wfk.visualize_ur2(spin, kpoint, band, appname=appname)
    #        thread = awx.WorkerThread(self, target=visu)
    #        thread.start()
    #
    #    except:
    #        awx.showErrorMessage(self)

    @property
    def viewer_frame(self):
        """The parent frame `WfkViewerFrame`."""
        try:
            return self._viewer_frame

        except AttributeError:
            self._viewer_frame = self.getParentWithType(WfkViewerFrame)
            return self._viewer_frame


class WfkViewerApp(awx.App):
    pass


def wxapp_wfkviewer(wfk_filepaths):
    app = WfkViewerApp()
    frame = WfkViewerFrame(None, filepaths=wfk_filepaths)
    app.SetTopWindow(frame)
    frame.Show()
    return app




#!/usr/bin/env python
"""MANDELBROT SET"""

import numpy as np

def mandelbrot(extent=None, ndivs=(1000,1000), iterations=100):
    """
    Based on http://cyrille.rossant.net/mandelbrot-set/

    Args:
        extent:
            (x_min, x_max, y_min, y_max))
        (nx,ny):
            Number of points sampled along x and y.
        iterations:
            number of iterations
    """
    if extent is None:
        extent = (-2, 1, -1.5, 1.5)

    x_min, x_max, y_min, y_max = extent
    nx, ny = ndivs

    # x, y are matrices containing the real and imaginary parts of all z values in the grid
    xvals = np.linspace(x_min, x_max, nx)
    yvals = np.linspace(y_min, y_max, ny)

    x, y = np.meshgrid(xvals, yvals)

    # we define c as c=x+iy, c is a nx x ny matrix.
    c = x + 1j*y

    # initially, z=c, we copy so that z and c are different objects in memory
    z = c.copy()

    # m is used to plot the fractal
    m = np.zeros((nx, ny))

    # iterations
    for n in range(iterations):
        print("Completed %d %%" % (100 * n/iterations))

        # indices of the numbers c such that |z(c)|<=10, with z = z_n
        indices = (np.abs(z) <= 10)

        # update z
        z[indices] = z[indices]**2 + c[indices]

        # update the values in m
        m[indices] = n

    return xvals, yvals, m


import wx
from wxmplot import ImageFrame, ImagePanel


class MyPanel(ImagePanel):
    """Pass"""
    def __init__(self, parent, **kwargs):
        super(MyPanel, self).__init__(parent, **kwargs)

    #def onLeftUp(self, event=None):
    #    super(MyPanel, self).onLeftUp(event=event)
    #    print("onLeftUp")
    #    print(self.zoom_ini)

    def zoom_leftup(self, event=None):
        super(MyPanel, self).zoom_leftup(event=event)
        print("zoom_leftup", self.zoom_lims[-1])

    #def redraw(self, col='int'):
    #    print("in redraw")
    #    super(MyPanel, self).redraw(col=col)

    def set_viewlimits(self, axes=None, autoscale=False):
        super(MyPanel, self).set_viewlimits(axes=None, autoscale=False)

        xmin, xmax = self.axes.get_xlim()
        ymin, ymax = self.axes.get_ylim()
        print(xmin, xmax)
        print(ymin, ymax)

        #x, y, m = mandelbrot(density=100)
        #self.clear()
        #self.display(m)

    #def lasso_leftup(self, event=None):
    #    """leftup event handler for lasso mode"""
    #    print("in lasso")

    #def zoom_motion(self, event=None):
    #    print("zoom_motion")
    #    super(MyPanel, self).zoom_motion(event=event)
    #    print(self.zoom_ini)


class FractalsFrame(ImageFrame):
    def __init__(self, parent, **kwargs):
        super(FractalsFrame, self).__init__(parent, **kwargs)

        ndivs = (1000,1000)
        x, y, m = mandelbrot(ndivs=ndvis)
        #print("x",x[0])
        #print("y",x[0])

        self.display(m) # x=x[0], y=x[0])

        self.panel.__class__ = MyPanel

"""
An example of how to use wx or wxagg in an application with the new
toolbar - comment out the setA_toolbar line for no toolbar
"""

# Used to guarantee to use at least Wx2.8
import wxversion
wxversion.ensureMinimal('2.8')



import matplotlib

from mandelbrot import mandelbrot
import matplotlib.pyplot as plt

# uncomment the following to use wx rather than wxagg
#matplotlib.use('WX')
#from matplotlib.backends.backend_wx import FigureCanvasWx as FigureCanvas

# comment out the following to use wx rather than wxagg
matplotlib.use('WXAgg')
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from matplotlib.backends.backend_wx import NavigationToolbar2Wx
from matplotlib.figure import Figure


class MyNavigationToolbar(NavigationToolbar2Wx):

    def __init__(self, frame):
        super(MyNavigationToolbar, self).__init__(frame.canvas)
        self.frame = frame

    def release_zoom(self, event):
        super(MyNavigationToolbar, self).release_zoom(event)
        print("in release_zoom",event)

        ax = self.canvas.figure.get_axes()[0]
        xmin, xmax = ax.get_xlim()
        ymin, ymax = ax.get_ylim()
        print("xlim",xmin, xmax)
        print("ylim",ymin, ymax)

        extent = (xmin, xmax, ymin, ymax)
        self.frame.imshow_fractal(extent=extent)

        self.canvas.draw()
        self.canvas.Refresh(eraseBackground=False)


class ConfPanel(wx.Panel):

    def __init__(self, parent, **kwargs):
        super(ConfPanel, self).__init__(parent, -1, **kwargs)
        self.BuildUi()

    def BuildUi(self):
        main_sizer = wx.BoxSizer(wx.VERTICAL)

        # Get a list of the colormaps in matplotlib.  Ignore the ones that end with
        # '_r' because these are simply reversed versions of ones that don't end with '_r'
        colormap_list = sorted(m for m in plt.cm.datad if not m.endswith("_r"))
        self.cmap_choice = wx.Choice(self, choices=colormap_list)

        main_sizer.Add(self.cmap_choice, 0, wx.LEFT, 5)

        self.SetSizerAndFit(main_sizer)


class CanvasFrame(wx.Frame):

    def __init__(self, parent):
        wx.Frame.__init__(self, parent, -1, 'CanvasFrame', size=(550,350))

        #main_sizer = wx.BoxSizer(wx.VERTICAL)
        #self.conf_panel = ConfPanel(self)
        #main_sizer.Add(self.conf_panel, 0, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 5)

        self.SetBackgroundColour(wx.NamedColour("WHITE"))
        self.figure = Figure()
        self.axes = self.figure.add_subplot(111)

        self.cmap = plt.cm.hot
        self.ndivs = (1000, 1000)

        self.canvas = FigureCanvas(self, -1, self.figure)

        self.sizer = sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.canvas, 1, wx.LEFT | wx.TOP | wx.GROW)

        self.toolbar = MyNavigationToolbar(self)
        self.toolbar.Realize()

        # On Windows platform, default window size is incorrect, so set toolbar width to figure width.
        tw, th = self.toolbar.GetSizeTuple()
        fw, fh = self.canvas.GetSizeTuple()
        # By adding toolbar in sizer, we are able to put it at the bottom
        # of the frame - so appearance is closer to GTK version.
        self.toolbar.SetSize(wx.Size(fw, th))
        sizer.Add(self.toolbar, 0, wx.LEFT | wx.EXPAND)

        # Update the axes menu on the toolbar
        self.toolbar.update()

        self.SetSizerAndFit(sizer)

        # Compute fractal and show picture.
        self.imshow_fractal()

    def imshow_fractal(self, extent=None):
        xvals, yvals, data = mandelbrot(extent=extent, ndivs=self.ndivs)

        if extent is None:
            extent = (xvals[0], xvals[-1], yvals[0], yvals[-1])

        self.axes.imshow(np.log(data), cmap=plt.cm.hot, origin="lower", extent=extent)

        # Save values
        self.xvals, self.yvals, self.data = xvals, yvals, data

    #def onclick(self, event):
    #    print('button=%d, x=%d, y=%d, xdata=%f, ydata=%f'%(
    #          event.button, event.x, event.y, event.xdata, event.ydata))


def wxapp_fractals():
    app = wx.App()
    #frame = FractalsFrame(None)
    frame = CanvasFrame(None)
    app.SetTopWindow(frame)
    frame.Show()
    return app


if __name__ == "__main__":
    import sys
    wxapp_fractals().MainLoop()
    sys.exit(0)

    x, y, m = mandelbrot()

    # we plot log(m)
    import matplotlib.pyplot as plt
    fig = plt.imshow(np.log(m), cmap=plt.cm.hot, extent=extent)

    plt.title('Mandelbrot Set')
    plt.xlabel('Re(z)')
    plt.ylabel('Im(z)')

    plt.show()



"""This module import all WxWidgets applications provides by Abipy in a single namespace."""

from .events import wxapp_events
from .browser import wxapp_dirbrowser, wxapp_listbrowser
from .scissors import wxapp_scissors
from .wfkviewer import wxapp_wfkviewer
from .sigresviewer import wxapp_sigresviewer
from .comparison import wxapp_comparison
from .editor import wxapp_showfiles
from .converter import wxapp_converter
from .structure import wxapp_structure_converter
from .fftprof import wxapp_fftprof
from .flowviewer import wxapp_flow_viewer
from .gsrviewer import wxapp_gsrviewer
from .mdfviewer import wxapp_mdfviewer
from .wxncview import wxapp_ncview
#from .oncvgui import wxapp_oncvpsp
from .phbstviewer import wxapp_phbstviewer


# Map abinit file extensions to WX Applications.
_EXT2APP = {
    "WFK-etsf.nc": wxapp_wfkviewer,
    "SIGRES.nc": wxapp_sigresviewer,
    "GSR.nc": wxapp_gsrviewer,
    "MDF.nc": wxapp_mdfviewer,
    "PHBST.nc": wxapp_phbstviewer,
    #".abi": MyEditorApp,
    #".abo": MyEditorApp,
    #".log": MyEditorApp,
    #".sh": MyEditorApp,
    #".err": MyEditorApp,
    #".files": MyEditorApp,
}


def file2appcls(filepath):
    import os
    ext = filepath.split("_")[-1]
    try:
        return _EXT2APP[ext]

    except KeyError:
        root, ext = os.path.splitext(filepath)
        try:
            return _EXT2APP[ext]
        except KeyError:
            # No frame registered for the file.
            # Open NcViewer if we have a netcdf file else None
            if filepath.endswith(".nc"): return wxapp_ncview
            return None



import os
import wx
import wx.lib.agw.flatnotebook as fnb
import abipy.gui.awx as awx

from monty.string import  marquee
from monty.collections import AttrDict
from wx.py.shell import Shell
from abipy.abilab import abiopen
from abipy.electrons import SigresPlotter
from abipy.gui.scissors import ScissorsBuilderFrame
from abipy.gui.baseviewer import MultiViewerFrame
from abipy.gui import mixins as mix
from abipy.gui.kpoints import SpinKpointBandPanel


class SigresViewerFrame(MultiViewerFrame, mix.Has_Structure, mix.Has_MultipleEbands, mix.Has_Tools, mix.Has_NetcdfFiles):
    VERSION = "0.1"

    HELP_MSG = """Quick help:

 Kpoint list:

     Left-Click:   to visualize table with QP results for the selected spin, k-point
     Right-Click:  display popup menu with choices.

Also, these key bindings can be used
(For Mac OSX, replace 'Ctrl' with 'Apple'):

  Ctrl-Q:     quit
"""

    @property
    def codename(self):
        return "SigmaResultsViewer"

    @property
    def active_sigres(self):
        """The active SIGRES file i.e. the SIGRES associated to the active tab."""
        return self.active_tab.sigres

    @property
    def structure(self):
        """`Structure` associated to the active tab."""
        return self.active_sigres.structure

    @property
    def ebands(self):
        """The electron bands associated to the active SIGRES file."""
        return self.active_sigres.ebands

    @property
    def ebands_list(self):
        """List of `ElectronBands`."""
        ebands_list = []
        for page in range(self.notebook.GetPageCount()):
            tab = self.notebook.GetPage(page)
            ebands_list.append(tab.sigres.ebands)
        return ebands_list

    @property
    def ebands_filepaths(self):
        """
        Return a list with the absolute paths of the files
        from which the `ElectronBands` have been read.
        """
        paths = []
        for page in range(self.notebook.GetPageCount()):
            tab = self.notebook.GetPage(page)
            paths.append(tab.sigres.filepath)
        return paths

    @property
    def nc_filepaths(self):
        """String with the absolute path of the netcdf file."""
        paths = []
        for page in range(self.notebook.GetPageCount()):
            tab = self.notebook.GetPage(page)
            paths.append(tab.sigres.filepath)
        return paths

    @property
    def sigres_filepaths(self):
        """
        Return a list with the absolute paths of the files
        from which the `SigresFile` have been read.
        """
        paths = []
        for page in range(self.notebook.GetPageCount()):
            tab = self.notebook.GetPage(page)
            paths.append(tab.sigres.filepath)
        return paths

    def makeMenu(self):
        """Creates the main menu."""
        # Base menu
        menu_bar = super(SigresViewerFrame, self).makeMenu()

        # Add mixin menus.
        menu_bar.Append(self.CreateStructureMenu(), "Structure")
        menu_bar.Append(self.CreateEbandsMenu(), "Ebands")
        menu_bar.Append(self.CreateQpDataMenu(), "QP Data")
        menu_bar.Append(self.CreateToolsMenu(), "Tools")
        menu_bar.Append(self.CreateNetcdfMenu(), "Netcdf")

        # Help menu
        help_menu = self.makeHelpMenu()
        menu_bar.Append(help_menu, "Help")

        self.SetMenuBar(menu_bar)

    def CreateQpDataMenu(self):
        """Build and return the QP menu."""
        # Structure Menu ID's
        self.ID_QPGAPS_COMPARE = wx.NewId()
        self.ID_QPENES_COMPARE = wx.NewId()

        menu = wx.Menu()
        menu.Append(self.ID_QPGAPS_COMPARE, "Compare QP-gaps", "Compare QP gaps")
        self.Bind(wx.EVT_MENU, self.OnQpGapsCompare, id=self.ID_QPGAPS_COMPARE)

        menu.Append(self.ID_QPENES_COMPARE, "Compare QP-enes", "Compare QP energies")
        self.Bind(wx.EVT_MENU, self.OnQpEnesCompare, id=self.ID_QPENES_COMPARE)

        return menu

    def OnQpGapsCompare(self, event):
        """Compare the QP gaps reported in the SIGRES files."""
        # Instanciate the plotter and add the filepaths to the plotter.
        plotter = SigresPlotter()
        plotter.add_files(self.sigres_filepaths)

        # Plot the convergence of the QP gaps.
        plotter.plot_qpgaps(title="Convergence of QP gaps", hspan=0.05)

    def OnQpEnesCompare(self, event):
        """Compare the QP energies reported in the SIGRES files."""
        # Instanciate the plotter and add the filepaths to the plotter.
        plotter = SigresPlotter()
        plotter.add_files(self.sigres_filepaths)

        # Plot the convergence of the QP energies.
        plotter.plot_qpenes(title="Convergence of QP energies", hspan=0.05)

    def makeToolBar(self):
        """Creates the toolbar."""
        self.toolbar = toolbar = self.CreateToolBar()
        toolbar.SetToolBitmapSize(wx.Size(48, 48))

        def bitmap(path):
            return wx.Bitmap(awx.path_img(path))

        self.ID_SCISSORS = wx.NewId()
        self.ID_PLOTQPSE0 = wx.NewId()
        self.ID_PLOTKSWITHMARKS = wx.NewId()

        artBmp = wx.ArtProvider.GetBitmap
        #toolbar.AddSimpleTool(wx.ID_OPEN, artBmp(wx.ART_FILE_OPEN, wx.ART_TOOLBAR), "Open")
        toolbar.AddSimpleTool(self.ID_PLOTQPSE0, bitmap("qpresults.png"), "Plot QPState Results.")
        toolbar.AddSimpleTool(self.ID_PLOTKSWITHMARKS, bitmap("qpmarkers.png"), "Plot KS energies with QPState markers.")
        toolbar.AddSimpleTool(self.ID_SCISSORS, bitmap("qpscissor.png"), "Build energy-dependent scissors from GW correction.")

        # Associate menu/toolbar items with their handlers.
        menu_handlers = [
            (self.ID_PLOTQPSE0, self.OnPlotQpsE0),
            (self.ID_PLOTKSWITHMARKS, self.OnPlotKSwithQPmarkers),
            (self.ID_SCISSORS, self.OnScissors),
        ]

        for combo in menu_handlers:
            mid, handler = combo[:2]
            self.Bind(wx.EVT_MENU, handler, id=mid)

        self.toolbar.Realize()

    def addFileTab(self, parent, filepath):
        """Open a SIGRES file and create a new tab in the notebook."""
        gsr = abiopen(filepath)
        tab = SigresFileTab(self.notebook, gsr)
        self.notebook.AddPage(tab, os.path.basename(filepath))

    def OnPlotQpsE0(self, event):
        """Plot QPState results as function of the KS energy."""
        if self.active_sigres is None: return
        self.active_sigres.plot_qps_vs_e0()

    def OnPlotKSwithQPmarkers(self, event):
        """Plot KS energies with QPState markers."""
        if self.active_sigres is None: return
        sigres = self.active_sigres

        band_range = (sigres.min_gwbstart, sigres.max_gwbstop)
        try:
            EbandsWithMarkersPlotFrame(self, sigres.ebands, band_range=band_range).Show()
        except:
            awx.showErrorMessage(self)

    def OnScissors(self, event):
        """Build the scissors operator."""
        if self.active_sigres is None: return
        ScissorsBuilderFrame(self, self.active_sigres.filepath).Show()



class SigresFileTab(wx.Panel):
    """Tab showing information on a single SIGRES file."""
    def __init__(self, parent, sigres, **kwargs):
        """
        Args:
            parent:
                parent window.
            sigres:
                `SigresFile` object
        """
        super(SigresFileTab, self).__init__(parent, -1, **kwargs)
        self.sigres = sigres

        splitter = wx.SplitterWindow(self, style=wx.SP_LIVE_UPDATE)
        splitter.SetSashGravity(0.95)

        self.skb_panel = SpinKpointBandPanel(splitter, sigres.structure, sigres.nsppol, sigres.gwkpoints, sigres.max_gwbstop,
                                             bstart=sigres.min_gwbstart)

        # Set the callback for double click on k-point row..
        self.Bind(self.skb_panel.MYEVT_SKB_ACTIVATED, self.onShowQPTable)

        # Add Python shell
        msg = "SIGRES_File object is accessible via the sigres variable. Use sigres.<TAB> to access the list of methods."
        msg = marquee(msg, width=len(msg) + 8, mark="#")
        msg = "#"*len(msg) + "\n" + msg + "\n" + "#"*len(msg) + "\n"

        pyshell = Shell(splitter, introText=msg, locals={"sigres": sigres})
        splitter.SplitHorizontally(self.skb_panel, pyshell)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(splitter, 1, wx.EXPAND, 5)
        self.SetSizerAndFit(sizer)

    @property
    def statusbar(self):
        return self.viewer_frame.statusbar

    @property
    def viewer_frame(self):
        """The parent frame `WfkViewerFrame`."""
        try:
            return self._viewer_frame

        except AttributeError:
            self._viewer_frame = self.getParentWithType(SigresViewerFrame)
            return self._viewer_frame

    def onShowQPTable(self, event):
        """Show a table with the QP results for the selected spin, kpoint, band."""
        spin, kpoint, band = event.skb
        qplist = self.sigres.get_qplist(spin, kpoint)
        table = qplist.to_table()
        title = "spin: %d, kpoint: %s" % (spin, kpoint)

        awx.SimpleGridFrame(self, table, labels_from_table=True, title=title).Show()


class EbandsWithMarkersPlotFrame(awx.Frame):

    def __init__(self, parent, ebands, **kwargs):
        """
        Args:
            parent:
                Parent window
            ebands:
                `ElectronBands` object.
        """
        self.band_range = kwargs.pop("band_range", None)

        super(EbandsWithMarkersPlotFrame, self).__init__(parent, -1, **kwargs)
        self.SetTitle("Select parameters")

        self.ebands = ebands

        if not ebands.markers:
            raise awx.Error("Found empty markers dictionary in ebands %s" % repr(ebands))

        self.BuildUi()

    def BuildUi(self):

        main_sizer = wx.BoxSizer(wx.VERTICAL)

        hsizer1 = wx.BoxSizer( wx.HORIZONTAL )

        marker_label = wx.StaticText(self, -1, "Available Markers:")
        marker_label.Wrap(-1)
        hsizer1.Add(marker_label, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        marker_choices = list(self.ebands.markers.keys())
        self.marker_choice = wx.Choice( self, -1, wx.DefaultPosition, wx.DefaultSize, marker_choices, 0 )
        self.marker_choice.SetSelection(0)
        hsizer1.Add(self.marker_choice, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        scale_label = wx.StaticText(self, -1, "Scale Factor:")
        scale_label.Wrap(-1)
        hsizer1.Add(scale_label, 0, wx.ALIGN_CENTER_VERTICAL|wx.TOP|wx.BOTTOM|wx.LEFT, 5 )

        self.scale_ctrl = wx.SpinCtrlDouble(self, id=-1, value=str(50), min=0.0, max=1.e+10, inc=50)
        self.scale_ctrl.SetToolTipString("Multiplicative factor used to render the markers more visible.")
        hsizer1.Add( self.scale_ctrl, 0, wx.ALL, 5 )

        main_sizer.Add( hsizer1, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )

        hsizer2 = wx.BoxSizer( wx.HORIZONTAL )

        ok_button = wx.Button(self, wx.ID_OK, label='Ok')
        cancel_button = wx.Button(self, wx.ID_CANCEL, label='Cancel')

        ok_button.Bind(wx.EVT_BUTTON, self.OnOkButton)
        cancel_button.Bind(wx.EVT_BUTTON, self.OnCloseButton)

        hsizer2.Add(ok_button, 0, wx.ALL, 5 )
        hsizer2.Add(cancel_button, 0, wx.ALL, 5 )

        main_sizer.Add( hsizer2, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )

        self.SetSizerAndFit(main_sizer)

    def OnCloseButton(self, event):
        self.Destroy()

    def GetParams(self):
        return AttrDict(
            qpattr=self.marker_choice.GetStringSelection(),
            fact=float(self.scale_ctrl.GetValue()))

    def OnOkButton(self, event):
        p = self.GetParams()
        with_marker = p.qpattr + ":" + str(p.fact)

        self.ebands.plot(marker=with_marker, band_range=self.band_range)


class SigresViewerApp(awx.App):
    pass


def wxapp_sigresviewer(sigres_filepaths):
    app = SigresViewerApp()
    frame = SigresViewerFrame(None, filepaths=sigres_filepaths)
    app.SetTopWindow(frame)
    frame.Show()
    return app



#!/usr/bin/env python
import wx

import abipy.data as abidata

from abipy.abilab import abiopen
from abipy.gui.kpoints import KpointsPanel

gsr = abiopen(abidata.ref_file("si_nscf_GSR.nc"))

app = wx.App()
frame = wx.Frame(None, -1)
KpointsPanel(frame, gsr.structure, gsr.kpoints)
app.SetTopWindow(frame)
frame.Show()
app.MainLoop()
gsr.close()
